package edu.uoc.android.gfp;

import android.app.Application;
import android.support.annotation.NonNull;
import android.widget.ArrayAdapter;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;
import com.orm.SugarContext;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


import edu.uoc.android.gfp.modeloDatos.T_Contribucion;
import edu.uoc.android.gfp.modeloDatos.T_Cuenta_Bancaria;
import edu.uoc.android.gfp.modeloDatos.T_Deseo;
import edu.uoc.android.gfp.modeloDatos.T_Gasto;
import edu.uoc.android.gfp.modeloDatos.T_Ingreso;
import edu.uoc.android.gfp.modeloDatos.T_Periodicidad;
import edu.uoc.android.gfp.modeloDatos.T_Presupuesto;
import edu.uoc.android.gfp.modeloDatos.T_Situacion;
import edu.uoc.android.gfp.modeloDatos.T_Usuario;
import edu.uoc.android.gfp.modeloDatos.T_tipo_Alimentacion;
import edu.uoc.android.gfp.modeloDatos.T_tipo_Gasto;

import edu.uoc.android.gfp.modeloDatos.T_tipo_Ingreso;
import edu.uoc.android.gfp.modeloDatos.T_tipo_Necesidad;
import edu.uoc.android.gfp.modeloDatos.T_tipo_Periodicidad;

public class MyApplication extends Application {

    //Propiedad para conectarse a Firebase de google
    public FirebaseAuth mAuth;
    public FirebaseDatabase database;

    //Variable que utilizamos para indicar si hay conexión a Internet
    public boolean conexOnline;

    //Propiedad para BD
    public DatabaseReference objBD;

    public void onCreate() {
        super.onCreate();
        SugarContext.init(this.getApplicationContext());

        /*-------------------------------------------------------------------
        ---------------------------------------------------------------------
        -------------INFO RELATIVA A LA CONEXIÓN CON LA BD ------------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
         */

        conexOnline = false;

        mAuth = FirebaseAuth.getInstance();
        database = FirebaseDatabase.getInstance();

        //Autenticación contra la BD de firebase
        mAuth.signInWithEmailAndPassword("fgg2@alu.ua.es", "12345678")
                .addOnCompleteListener(new OnCompleteListener<AuthResult>() {
                    @Override
                    public void onComplete(@NonNull Task<AuthResult> task) {

                        if (!task.isSuccessful()) {
                            //Si no hay conexión con el servidor enviamos una notificación a la app
                            //Indicando que la señal ha fallado
                            conexOnline = false;
                        } else
                            conexOnline = true;
                    }
                });

        //Actualiza la información relativa a la BD
        actualizaInfoBDLocal();

    }








        /*-------------------------------------------------------------------
        ---------------------------------------------------------------------
        -------------SINCRONIZA LA INFORMACIÓN DE FB CON LA BD LOCAL --------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
         */

    public void actualizaInfoBDLocal() {

        // GASTOS

        objBD = database.getReference("Gasto");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                ArrayList<T_Gasto.Gasto> values = new ArrayList<T_Gasto.Gasto>();

                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    values.add(child.getValue(T_Gasto.Gasto.class));
                }

                Iterator<T_Gasto.Gasto> it = values.iterator();
                T_Gasto.Gasto gasto_it;
                // mientras queden gastos
                while (it.hasNext()) {
                    gasto_it = it.next();

                    if (gasto_it != null) {
                        //if (!T_Gasto.existe(gasto_it)) {
                        gasto_it.save();
                        //}
                    }
                }

            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


        // INGRESOS

        objBD = database.getReference("Ingreso");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                ArrayList<T_Ingreso.Ingreso> values = new ArrayList<T_Ingreso.Ingreso>();

                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    values.add(child.getValue(T_Ingreso.Ingreso.class));
                }

                Iterator<T_Ingreso.Ingreso> it = values.iterator();
                T_Ingreso.Ingreso ingreso_it;
                // mientras queden ingresos
                while (it.hasNext()) {
                    ingreso_it = it.next();

                    if (ingreso_it != null) {
                        //if (!T_Ingreso.existe(ingreso_it)) {
                        ingreso_it.save();
                        //}
                    }
                }

            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


        // PRESUPUESTOS

        objBD = database.getReference("Presupuesto");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                ArrayList<T_Presupuesto.Presupuesto> values = new ArrayList<T_Presupuesto.Presupuesto>();

                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    values.add(child.getValue(T_Presupuesto.Presupuesto.class));
                }

                Iterator<T_Presupuesto.Presupuesto> it = values.iterator();
                T_Presupuesto.Presupuesto presupuesto_it;
                // mientras queden PRESUPUESTOS
                while (it.hasNext()) {
                    presupuesto_it = it.next();

                    if (presupuesto_it != null) {
                        //if (!T_Presupuesto.existe(presupuesto_it)) {
                        presupuesto_it.save();
                        //}
                    }
                }

            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


        // DESEOS

        objBD = database.getReference("Deseo");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                ArrayList<T_Deseo.Deseo> values = new ArrayList<T_Deseo.Deseo>();

                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    values.add(child.getValue(T_Deseo.Deseo.class));
                }

                Iterator<T_Deseo.Deseo> it = values.iterator();
                T_Deseo.Deseo deseo_it;
                // mientras queden deseos
                while (it.hasNext()) {
                    deseo_it = it.next();

                    if (deseo_it != null) {
                        //if (!T_Deseo.existe(deseo_it)) {
                        deseo_it.save();
                        //}
                    }
                }

            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


        // SITUACION

        objBD = database.getReference("Situacion");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                ArrayList<T_Situacion.Situacion> values = new ArrayList<T_Situacion.Situacion>();

                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    values.add(child.getValue(T_Situacion.Situacion.class));
                }

                Iterator<T_Situacion.Situacion> it = values.iterator();
                T_Situacion.Situacion situacion_it;
                // mientras queden situaciones
                while (it.hasNext()) {
                    situacion_it = it.next();

                    if (situacion_it != null) {
                        //if (!T_Situacion.existe(situacion_it)) {
                        situacion_it.save();
                        //}
                    }
                }

            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


        // USUARIO

        objBD = database.getReference("Usuario");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                ArrayList<T_Usuario.Usuario> values = new ArrayList<T_Usuario.Usuario>();

                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    values.add(child.getValue(T_Usuario.Usuario.class));
                }

                Iterator<T_Usuario.Usuario> it = values.iterator();
                T_Usuario.Usuario usuario_it;
                // mientras queden usuarios
                while (it.hasNext()) {
                    usuario_it = it.next();

                    if (usuario_it != null) {
                        //if (!T_Usuario.existe(usuario_it)) {
                        usuario_it.save();
                        //}
                    }
                }

            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


        // TIPO_GASTO

        objBD = database.getReference("Tipo_Gasto");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                ArrayList<T_tipo_Gasto.Tipo_Gasto> values = new ArrayList<T_tipo_Gasto.Tipo_Gasto>();

                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    values.add(child.getValue(T_tipo_Gasto.Tipo_Gasto.class));
                }

                Iterator<T_tipo_Gasto.Tipo_Gasto> it = values.iterator();
                T_tipo_Gasto.Tipo_Gasto tgasto_it;
                // mientras queden tipos de gasto
                while (it.hasNext()) {
                    tgasto_it = it.next();

                    if (tgasto_it != null) {
                        //if (!T_tipo_Gasto.existe(tgasto_it)) {
                        tgasto_it.save();
                        //}
                    }
                }

            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });

        // TIPO_INGRESO

        objBD = database.getReference("Tipo_Ingreso");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                ArrayList<T_tipo_Ingreso.Tipo_Ingreso> values = new ArrayList<T_tipo_Ingreso.Tipo_Ingreso>();

                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    values.add(child.getValue(T_tipo_Ingreso.Tipo_Ingreso.class));
                }

                Iterator<T_tipo_Ingreso.Tipo_Ingreso> it = values.iterator();
                T_tipo_Ingreso.Tipo_Ingreso tingreso_it;
                // mientras queden tipos de ingreso
                while (it.hasNext()) {
                    tingreso_it = it.next();

                    if (tingreso_it != null) {
                        //if (!T_tipo_Ingreso.existe(tingreso_it)) {
                        tingreso_it.save();
                        //}
                    }
                }

            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });

        // TIPO_PERIODICIDAD

        objBD = database.getReference("Tipo_Periodicidad");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                ArrayList<T_tipo_Periodicidad.Tipo_Periodicidad> values = new ArrayList<T_tipo_Periodicidad.Tipo_Periodicidad>();

                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    values.add(child.getValue(T_tipo_Periodicidad.Tipo_Periodicidad.class));
                }

                Iterator<T_tipo_Periodicidad.Tipo_Periodicidad> it = values.iterator();
                T_tipo_Periodicidad.Tipo_Periodicidad periodicidad_it;
                // mientras queden tipos de gasto
                while (it.hasNext()) {
                    periodicidad_it = it.next();

                    if (periodicidad_it != null) {
                        //if (!T_tipo_Periodicidad.existe(periodicidad_it)) {
                        periodicidad_it.save();
                        //}
                    }
                }

            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


        // NECESIDAD
        objBD = database.getReference("Tipo_Necesidad");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                ArrayList<T_tipo_Necesidad.Tipo_Necesidad> values = new ArrayList<T_tipo_Necesidad.Tipo_Necesidad>();

                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    values.add(child.getValue(T_tipo_Necesidad.Tipo_Necesidad.class));
                }

                Iterator<T_tipo_Necesidad.Tipo_Necesidad> it = values.iterator();
                T_tipo_Necesidad.Tipo_Necesidad necesidad_it;
                // mientras queden tipos de necesidad
                while (it.hasNext()) {
                    necesidad_it = it.next();

                    if (necesidad_it != null) {
                        //if (!T_tipo_Necesidad.existe(necesidad_it)) {
                        necesidad_it.save();
                        //}
                    }
                }

            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


        // CUENTA BANCARIA
        objBD = database.getReference("Cuenta_Bancaria");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                ArrayList<T_Cuenta_Bancaria.Cuenta_Bancaria> values = new ArrayList<T_Cuenta_Bancaria.Cuenta_Bancaria>();

                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    values.add(child.getValue(T_Cuenta_Bancaria.Cuenta_Bancaria.class));
                }

                Iterator<T_Cuenta_Bancaria.Cuenta_Bancaria> it = values.iterator();
                T_Cuenta_Bancaria.Cuenta_Bancaria cuentaBancaria_it;
                // mientras queden tipos de necesidad
                while (it.hasNext()) {
                    cuentaBancaria_it = it.next();

                    if (cuentaBancaria_it != null) {
                        //if (!T_Cuenta_Bancaria.existe(cuentaBancaria_it)) {
                        cuentaBancaria_it.save();
                        //}
                    }
                }

            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


        // CONTRIBUCION
        objBD = database.getReference("Contribucion");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                ArrayList<T_Contribucion.Contribucion> values = new ArrayList<T_Contribucion.Contribucion>();

                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    values.add(child.getValue(T_Contribucion.Contribucion.class));
                }

                Iterator<T_Contribucion.Contribucion> it = values.iterator();
                T_Contribucion.Contribucion contribucion_it;
                // mientras queden contribuciones
                while (it.hasNext()) {
                    contribucion_it = it.next();

                    if (contribucion_it != null) {
                        //if (!T_Contribucion.existe(contribucion_it)) {
                        contribucion_it.save();
                        //}
                    }
                }

            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


        // TIPO_PERIODICIDAD
        objBD = database.getReference("Tipo_Periodicidad");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                ArrayList<T_tipo_Periodicidad.Tipo_Periodicidad> values = new ArrayList<T_tipo_Periodicidad.Tipo_Periodicidad>();

                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    values.add(child.getValue(T_tipo_Periodicidad.Tipo_Periodicidad.class));
                }

                Iterator<T_tipo_Periodicidad.Tipo_Periodicidad> it = values.iterator();
                T_tipo_Periodicidad.Tipo_Periodicidad periodicidad_it;
                // mientras queden contribuciones
                while (it.hasNext()) {
                    periodicidad_it = it.next();

                    if (periodicidad_it != null) {
                        //if (!T_tipo_Periodicidad.existe(periodicidad_it)) {
                        periodicidad_it.save();
                        //}
                    }
                }

            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


        // TIPO_ALIMENTACION

        objBD = database.getReference("Tipo_Alimentacion");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {

                ArrayList<T_tipo_Alimentacion.Tipo_Alimentacion> values = new ArrayList<T_tipo_Alimentacion.Tipo_Alimentacion>();

                for (DataSnapshot child : dataSnapshot.getChildren()) {
                    values.add(child.getValue(T_tipo_Alimentacion.Tipo_Alimentacion.class));
                }

                Iterator<T_tipo_Alimentacion.Tipo_Alimentacion> it = values.iterator();
                T_tipo_Alimentacion.Tipo_Alimentacion talimentacion_it;
                // mientras queden tipos de alimentacion
                while (it.hasNext()) {
                    talimentacion_it = it.next();

                    if (talimentacion_it != null) {
                        //if (!T_tipo_Alimentacion.existe(talimentacion_it)) {
                        talimentacion_it.save();
                        //}
                    }
                }

            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });

    }





        /*-------------------------------------------------------------------
        ---------------------------------------------------------------------
        -------------INFO RELATIVA AL OBJETO TIPO_GASTO... ------------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
         */


    //obtener Tipos de Gasto de la BD
    private ArrayList<String> obtenerTipoGastoBD() {

        List<T_tipo_Gasto.Tipo_Gasto> values = T_tipo_Gasto.obtenerTodosTiposGasto();
        ArrayList<String> tiposGastos = new ArrayList<String>();

        Iterator<T_tipo_Gasto.Tipo_Gasto> it = values.iterator();
        T_tipo_Gasto.Tipo_Gasto tipo_gasto_it;
        // mientras queden tipos de gasto
        while (it.hasNext()) {
            tipo_gasto_it = it.next();
            if (tipo_gasto_it != null) {
                tiposGastos.add(tipo_gasto_it.getdescripcion());
            }
        }

        return (tiposGastos);
    }

    //Carga desplegable de Necesidad
    public ArrayAdapter<String> cargaNecesidad() {
        ArrayAdapter<String> dataAdapter = null;

        //Si no hay conexión se obtiene de BD local
        ArrayList<String> necesidades = new ArrayList<String>();
        necesidades.add(getResources().getString(R.string.ini_spinner));

        List<T_tipo_Necesidad.Tipo_Necesidad> tn_bd = T_tipo_Necesidad.obtenerTodosTiposNecesidad();

        for (int indice = 0; indice < tn_bd.size(); indice++) {
            necesidades.add(tn_bd.get(indice).descripcion);
        }

        dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, necesidades);

        return dataAdapter;
    }

    //Carga desplegable de TipoGasto
    public ArrayAdapter<String> cargaTipoGasto() {
        ArrayAdapter<String> dataAdapter = null;

        //Si no hay conexión se obtiene de BD local
        ArrayList<String> tiposGastos = new ArrayList<String>();
        tiposGastos.add(getResources().getString(R.string.ini_spinner));

        List<T_tipo_Gasto.Tipo_Gasto> tp_bd = T_tipo_Gasto.obtenerTodosTiposGasto();

        for (int indice = 0; indice < tp_bd.size(); indice++) {
            tiposGastos.add(tp_bd.get(indice).descripcion);
        }

        dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, tiposGastos);

        return dataAdapter;
    }


    //Carga desplegable de TipoGasto
    public ArrayAdapter<String> cargaTipoGastoFB() {

        final ArrayList<String> tiposGastos = new ArrayList<String>();
        tiposGastos.add(getResources().getString(R.string.ini_spinner));

        objBD = database.getReference("Tipo_Gasto");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                GenericTypeIndicator<ArrayList<T_tipo_Gasto.Tipo_Gasto>> genericTypeIndicator =
                        new GenericTypeIndicator<ArrayList<T_tipo_Gasto.Tipo_Gasto>>() {
                        };

                ArrayList<T_tipo_Gasto.Tipo_Gasto> values = dataSnapshot.getValue(genericTypeIndicator);

                Iterator<T_tipo_Gasto.Tipo_Gasto> it = values.iterator();
                T_tipo_Gasto.Tipo_Gasto gasto_it;
                // mientras queden tipos de gasto
                while (it.hasNext()) {
                    gasto_it = it.next();

                    if (gasto_it != null) {
                        tiposGastos.add(gasto_it.getdescripcion());

                    }
                }


            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


        ArrayAdapter<String> dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, tiposGastos);

        return dataAdapter;
    }


    public Integer altaTipoGasto(String descripcion,Integer idNecesidad, String necesidad) {
        Integer idTipoGasto;

        idTipoGasto = T_tipo_Gasto.altaTipoGasto(getApplicationContext(), descripcion,idNecesidad,necesidad);
        if (conexOnline)
            altaTipoGastoFB(idTipoGasto,descripcion,idNecesidad,necesidad);

        return idTipoGasto;
    }


    public void altaTipoGastoFB(Integer idTipoGasto,String descripcion,Integer idNecesidad, String necesidad) {

        objBD = database.getReference("Tipo_Gasto");

        T_tipo_Gasto.Tipo_Gasto nTipoGasto = new T_tipo_Gasto.Tipo_Gasto (idTipoGasto, descripcion, idNecesidad,necesidad);
        String item = objBD.child(idTipoGasto.toString()).getKey();
        objBD.child(item).setValue(nTipoGasto);
    }

    public void modificaTipoGasto(Integer idTipoGasto, String descripcion,Integer idNecesidad,String necesidad) {

        T_tipo_Gasto.modificaTipoGasto(getApplicationContext(), idTipoGasto,descripcion,idNecesidad,necesidad);

        if (conexOnline)
            modificaTipoGastoFB(idTipoGasto, descripcion,idNecesidad, necesidad);

    }


    public void modificaTipoGastoFB(Integer idTipoGasto, String descripcion,Integer idNecesidad, String necesidad) {

        objBD = database.getReference("Tipo_Gasto");

        Map<String, T_tipo_Gasto.Tipo_Gasto> nTipoGasto = new HashMap<>();
        nTipoGasto.put(idTipoGasto.toString(), new T_tipo_Gasto.Tipo_Gasto(idTipoGasto, descripcion, idNecesidad, necesidad));

        objBD.setValue(nTipoGasto);
    }


    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO TIPO PERIODICIDAD --------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */


    //Carga desplegable de TipoPeriodicidad

    public ArrayAdapter<String> cargaTipoPeriodicidad() {
        ArrayAdapter<String> dataAdapter = null;

        ArrayList<String> tiposPeriodicidad = new ArrayList<String>();
        tiposPeriodicidad.add(getResources().getString(R.string.ini_spinner));

        List<T_tipo_Periodicidad.Tipo_Periodicidad> tp_bd = T_tipo_Periodicidad.obtenerTodosTiposPeriodicidad();

        for (int indice = 0; indice < tp_bd.size(); indice++) {
            tiposPeriodicidad.add(tp_bd.get(indice).descripcion);
        }

        dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, tiposPeriodicidad);


        return dataAdapter;
    }


    //Carga desplegable de TipoGasto
    public ArrayAdapter<String> cargaTipoPeriodicidadFB() {

        final ArrayList<String> tiposPeriodicidad = new ArrayList<String>();
        tiposPeriodicidad.add(getResources().getString(R.string.ini_spinner));

        objBD = database.getReference("Tipo_Periodicidad");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                GenericTypeIndicator<ArrayList<T_Periodicidad.Tipo_Periodicidad>> genericTypeIndicator =
                        new GenericTypeIndicator<ArrayList<T_Periodicidad.Tipo_Periodicidad>>() {
                        };

                ArrayList<T_Periodicidad.Tipo_Periodicidad> values = dataSnapshot.getValue(genericTypeIndicator);

                Iterator<T_Periodicidad.Tipo_Periodicidad> it = values.iterator();
                T_Periodicidad.Tipo_Periodicidad periodicidad_it;
                // mientras queden tipos de gasto
                while (it.hasNext()) {
                    periodicidad_it = it.next();

                    if (periodicidad_it != null) {
                        tiposPeriodicidad.add(periodicidad_it.getdescripcion());
                    }
                }


            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });

        ArrayAdapter<String> dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, tiposPeriodicidad);

        return dataAdapter;
    }



    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO GASTO --------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */


    public Integer altaGasto(String descripcion, Integer idTipoGasto, float importe, String fechaGasto, Integer idPeriodicidad, Integer idNecesidad, String idUsuario,
                             String tipoGasto, String periodicidad, String necesidad) {
        Integer idGasto;

        idGasto = T_Gasto.altaGasto(getApplicationContext(), descripcion, idTipoGasto, importe, fechaGasto, idPeriodicidad, idNecesidad, idUsuario,
                tipoGasto, periodicidad, necesidad);
        if (conexOnline)
            altaGastoFB(idGasto, descripcion, idTipoGasto, importe, fechaGasto, idPeriodicidad, idNecesidad, idUsuario, tipoGasto, periodicidad, necesidad);

        return idGasto;
    }


    public void altaGastoFB(Integer idGasto, String descripcion, Integer idTipoGasto, float importe, String fechaGasto, Integer idPeriodicidad, Integer idNecesidad, String idUsuario,
                            String tipoGasto, String periodicidad, String necesidad) {

        objBD = database.getReference("Gasto");

        //objBD.child(idGasto.toString()).push().setValue(new T_Gasto.Gasto(idGasto,  descripcion,  idTipoGasto,  importe,  fechaGasto,  idPeriodicidad,  idNecesidad,  idUsuario,tipoGasto,periodicidad,necesidad));

        //Map<String, T_Gasto.Gasto> nGasto = new HashMap<>();
        //nGasto.put(idGasto.toString(),new T_Gasto.Gasto(idGasto,  descripcion,  idTipoGasto,  importe,  fechaGasto,  idPeriodicidad,  idNecesidad,  idUsuario,tipoGasto,periodicidad,necesidad));
        //objBD.setValue(nGasto);


        //objBD.push(idGasto.toString(),new T_Gasto.Gasto(idGasto,  descripcion,  idTipoGasto,  importe,  fechaGasto,  idPeriodicidad,  idNecesidad,  idUsuario,tipoGasto,periodicidad,necesidad));

        //objBD.child(idGasto.toString()).push().setValue(new T_Gasto.Gasto(idGasto,  descripcion,  idTipoGasto,  importe,  fechaGasto,  idPeriodicidad,  idNecesidad,  idUsuario,tipoGasto,periodicidad,necesidad));

        T_Gasto.Gasto nGasto = new T_Gasto.Gasto(idGasto, descripcion, idTipoGasto, importe, fechaGasto, idPeriodicidad, idNecesidad, idUsuario, tipoGasto, periodicidad, necesidad);
        String item = objBD.child(idGasto.toString()).getKey();
        objBD.child(item).setValue(nGasto);
    }

    public void modificaGasto(Integer idGasto, String descripcion, Integer idTipoGasto, float importe, String fechaGasto, Integer idPeriodicidad, Integer idNecesidad, String idUsuario,
                              String tipoGasto, String periodicidad, String necesidad) {

        T_Gasto.modificaGasto(getApplicationContext(), idGasto, descripcion, idTipoGasto, importe, fechaGasto, idPeriodicidad, idNecesidad, idUsuario,
                tipoGasto, periodicidad, necesidad);

        if (conexOnline)
            modificaGastoFB(idGasto, descripcion, idTipoGasto, importe, fechaGasto, idPeriodicidad, idNecesidad, idUsuario, tipoGasto, periodicidad, necesidad);

    }


    public void modificaGastoFB(Integer idGasto, String descripcion, Integer idTipoGasto, float importe, String fechaGasto, Integer idPeriodicidad, Integer idNecesidad, String idUsuario,
                                String tipoGasto, String periodicidad, String necesidad) {
        objBD = database.getReference("Gasto");

        Map<String, T_Gasto.Gasto> nGasto = new HashMap<>();
        nGasto.put(idGasto.toString(), new T_Gasto.Gasto(idGasto, descripcion, idTipoGasto, importe, fechaGasto, idPeriodicidad, idNecesidad, idUsuario, tipoGasto, periodicidad, necesidad));

        objBD.setValue(nGasto);
    }


    public void bajaGasto(Integer idGasto) {

        //Se elimina de la base de datos Local y tambien se elimina de FB
        T_Gasto.eliminarGasto(getApplicationContext(), idGasto);

        //Se elimina el gasto de FB también
        bajaGastoFB(idGasto);

    }


    public void bajaGastoFB(Integer idGasto) {
        //Eliminar de la BD de FB el gasto indicado
        objBD = database.getReference("Gasto");


        objBD.child(idGasto.toString()).removeValue();

    }



        /*-------------------------------------------------------------------
        ---------------------------------------------------------------------
        -------------INFO RELATIVA AL OBJETO TIPO_INGRESO...-----------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
         */


    private ArrayList<String> obtenerTipoIngresoBD() {

        List<T_tipo_Ingreso.Tipo_Ingreso> values = T_tipo_Ingreso.obtenerTodosTiposIngresos();
        ArrayList<String> tiposIngresos = new ArrayList<String>();

        Iterator<T_tipo_Ingreso.Tipo_Ingreso> it = values.iterator();
        T_tipo_Ingreso.Tipo_Ingreso tipo_ingreso_it;
        // mientras queden tipos de ingresos
        while (it.hasNext()) {
            tipo_ingreso_it = it.next();
            if (tipo_ingreso_it != null) {
                tiposIngresos.add(tipo_ingreso_it.getdescripcion());
            }
        }

        return (tiposIngresos);
    }

    //Carga desplegable de TipoIngreso
    public ArrayAdapter<String> cargaTipoIngreso() {
        ArrayAdapter<String> dataAdapter = null;

        //Si no hay conexión se obtiene de BD local
        ArrayList<String> tiposIngresos = new ArrayList<String>();
        tiposIngresos.add(getResources().getString(R.string.ini_spinner));

        List<T_tipo_Ingreso.Tipo_Ingreso> ti_bd = T_tipo_Ingreso.obtenerTodosTiposIngresos();

        for (int indice = 0; indice < ti_bd.size(); indice++) {
            tiposIngresos.add(ti_bd.get(indice).descripcion);
        }

        dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, tiposIngresos);

        return dataAdapter;
    }


    //Carga desplegable de TipoGasto
    public ArrayAdapter<String> cargaTipoIngresoFB() {

        final ArrayList<String> tiposIngresos = new ArrayList<String>();
        tiposIngresos.add(getResources().getString(R.string.ini_spinner));

        objBD = database.getReference("Tipo_Ingreso");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                GenericTypeIndicator<ArrayList<T_tipo_Ingreso.Tipo_Ingreso>> genericTypeIndicator =
                        new GenericTypeIndicator<ArrayList<T_tipo_Ingreso.Tipo_Ingreso>>() {
                        };

                ArrayList<T_tipo_Ingreso.Tipo_Ingreso> values = dataSnapshot.getValue(genericTypeIndicator);

                Iterator<T_tipo_Ingreso.Tipo_Ingreso> it = values.iterator();
                T_tipo_Ingreso.Tipo_Ingreso ingreso_it;


                while (it.hasNext()) {
                    ingreso_it = it.next();

                    if (ingreso_it != null) {
                        tiposIngresos.add(ingreso_it.getdescripcion());

                    }
                }


            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


        ArrayAdapter<String> dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, tiposIngresos);

        return dataAdapter;
    }


    public Integer altaTipoIngreso(String descripcion) {
        Integer idTipoIngreso;

        idTipoIngreso = T_tipo_Ingreso.altaTipoIngreso(getApplicationContext(), descripcion);
        if (conexOnline)
            altaTipoIngresoFB(idTipoIngreso,descripcion);

        return idTipoIngreso;
    }


    public void altaTipoIngresoFB(Integer idTipoIngreso,String descripcion) {

        objBD = database.getReference("Tipo_Ingreso");

        T_tipo_Ingreso.Tipo_Ingreso nTipoIngreso = new T_tipo_Ingreso.Tipo_Ingreso (idTipoIngreso, descripcion);
        String item = objBD.child(idTipoIngreso.toString()).getKey();
        objBD.child(item).setValue(nTipoIngreso);
    }

    public void modificaTipoIngreso(Integer idTipoIngreso, String descripcion) {

        T_tipo_Ingreso.modificaTipoIngreso(getApplicationContext(), idTipoIngreso,descripcion);

        if (conexOnline)
          modificaTipoIngresoFB(idTipoIngreso, descripcion);

    }


    public void modificaTipoIngresoFB(Integer idTipoIngreso, String descripcion) {

        objBD = database.getReference("Tipo_Ingreso");

        Map<String, T_tipo_Ingreso.Tipo_Ingreso> nTipoIngreso = new HashMap<>();
        nTipoIngreso.put(idTipoIngreso.toString(), new T_tipo_Ingreso.Tipo_Ingreso(idTipoIngreso, descripcion));

        objBD.setValue(nTipoIngreso);
    }


    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO INGRESO --------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */


    public void bajaIngreso(Integer idIngreso) {

        //Se elimina de la base de datos Local y tambien se elimina de FB
        T_Ingreso.eliminarIngreso(getApplicationContext(), idIngreso);

        if (conexOnline)
            //Se elimina el gasto de FB también
            bajaIngresoFB(idIngreso);

    }


    public void bajaIngresoFB(Integer idIngreso) {
        //Eliminar de la BD de FB el deseo indicado
        objBD = database.getReference("Ingreso");


        objBD.child(idIngreso.toString()).removeValue();


    }

    public Integer altaIngreso(String descripcion, Integer idTipoIngreso, float importe, String fechaIngreso, Integer idPeriodicidad, String idUsuario,
                               String tipoIngreso, String periodicidad) {
        Integer idIngreso;

        idIngreso = T_Ingreso.altaIngreso(getApplicationContext(), descripcion, idTipoIngreso, importe, fechaIngreso, idPeriodicidad, idUsuario,
                tipoIngreso, periodicidad);

        if (conexOnline)
            altaIngresoFB(idIngreso, descripcion, idTipoIngreso, importe, fechaIngreso, idPeriodicidad, idUsuario, tipoIngreso, periodicidad);

        return idIngreso;
    }


    public void altaIngresoFB(Integer idIngreso, String descripcion, Integer idTipoIngreso, float importe, String fechaIngreso, Integer idPeriodicidad, String idUsuario,
                              String tipoIngreso, String periodicidad) {
        objBD = database.getReference("Ingreso");

        T_Ingreso.Ingreso nIngreso = new T_Ingreso.Ingreso(idIngreso, descripcion, idTipoIngreso, importe, fechaIngreso, idPeriodicidad, idUsuario, tipoIngreso, periodicidad);
        String item = objBD.child(idIngreso.toString()).getKey();
        objBD.child(item).setValue(nIngreso);

    }

    public void modificaIngreso(Integer idIngreso, String descripcion, Integer idTipoIngreso, float importe, String fechaIngreso, Integer idPeriodicidad, String idUsuario,
                                String tipoIngreso, String periodicidad) {

        T_Ingreso.modificaIngreso(getApplicationContext(), idIngreso, descripcion, idTipoIngreso, importe, fechaIngreso, idPeriodicidad, idUsuario,
                tipoIngreso, periodicidad);

        if (conexOnline)
            modificaIngresoFB(idIngreso, descripcion, idTipoIngreso, importe, fechaIngreso, idPeriodicidad, idUsuario, tipoIngreso, periodicidad);

    }


    public void modificaIngresoFB(Integer idIngreso, String descripcion, Integer idTipoIngreso, float importe, String fechaIngreso, Integer idPeriodicidad, String idUsuario,
                                  String tipoIngreso, String periodicidad) {
        objBD = database.getReference("Ingreso");

        Map<String, T_Ingreso.Ingreso> nIngreso = new HashMap<>();
        nIngreso.put(idIngreso.toString(), new T_Ingreso.Ingreso(idIngreso, descripcion, idTipoIngreso, importe, fechaIngreso, idPeriodicidad, idUsuario, tipoIngreso, periodicidad));

        objBD.setValue(nIngreso);
    }



    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO CONTRIBUCION--------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */


    public void bajaContribucion(Integer idContribucion) {

        //Se elimina de la base de datos Local y tambien se elimina de FB
        T_Contribucion.eliminarContribucion(getApplicationContext(), idContribucion);

        if (conexOnline)
            //Se elimina el gasto de FB también
            bajaContribucionFB(idContribucion);

    }


    public void bajaContribucionFB(Integer idContribucion) {
        //Eliminar de la BD de FB contribucion
        objBD = database.getReference("Contribucion");


        objBD.child(idContribucion.toString()).removeValue();


    }

    public Integer altaContribucion(String descripcion, Integer idDeseo, float importe, String fechaContribucion, String idUsuario,
                                    String deseo) {
        Integer idContribucion;

        idContribucion = T_Contribucion.altaContribucion(getApplicationContext(), descripcion, idDeseo, importe, fechaContribucion, idUsuario,
                deseo);

        if (conexOnline)
            altaContribucionFB(idContribucion, descripcion, idDeseo, importe, fechaContribucion, idUsuario, deseo);

        return idContribucion;
    }


    public void altaContribucionFB(Integer idContribucion, String descripcion, Integer idDeseo, float importe, String fechaContribucion, String idUsuario,
                                   String deseo) {
        objBD = database.getReference("Contribucion");

        T_Contribucion.Contribucion nContribucion = new T_Contribucion.Contribucion(idContribucion, descripcion, importe, idDeseo, fechaContribucion, idUsuario, deseo);
        String item = objBD.child(idContribucion.toString()).getKey();
        objBD.child(item).setValue(nContribucion);

    }

    public void modificaContribucion(Integer idContribucion, String descripcion, Integer idDeseo, float importe, String fechaContribucion, String idUsuario,
                                     String deseo) {

        T_Contribucion.modificaContribucion(getApplicationContext(), idContribucion, descripcion, idDeseo, importe, fechaContribucion, idUsuario,
                deseo);

        if (conexOnline)
            modificaContribucionFB(idContribucion, descripcion, idDeseo, importe, fechaContribucion, idUsuario, deseo);

    }


    public void modificaContribucionFB(Integer idContribucion, String descripcion, Integer idDeseo, float importe, String fechaContribucion, String idUsuario,
                                       String deseo) {
        objBD = database.getReference("Contribucion");

        Map<String, T_Contribucion.Contribucion> nContribucion = new HashMap<>();
        nContribucion.put(idContribucion.toString(), new T_Contribucion.Contribucion(idContribucion, descripcion, importe, idDeseo, fechaContribucion, idUsuario, deseo));

        objBD.setValue(nContribucion);
    }




    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO DESEO---------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */


    //Carga desplegable de Deseo
    public ArrayAdapter<String> cargaDeseo(String idUsuario) {
        ArrayAdapter<String> dataAdapter = null;

        //Si no hay conexión se obtiene de BD local
        ArrayList<String> deseos = new ArrayList<String>();
        deseos.add(getResources().getString(R.string.ini_spinner));

        List<T_Deseo.Deseo> deseos_bd = T_Deseo.obtenerTodosDeseos(idUsuario);

        for (int indice = 0; indice < deseos_bd.size(); indice++) {
            deseos.add(deseos_bd.get(indice).descripcion);
        }

        dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, deseos);

        return dataAdapter;
    }


    public Integer altaDeseo(String descripcion, float importe, String desdeFecha, String hastaFecha, String idUsuario) {
        Integer idDeseo;

        idDeseo = T_Deseo.altaDeseo(getApplicationContext(), descripcion, importe, desdeFecha, hastaFecha, idUsuario);

        if (conexOnline)
            altaDeseoFB(idDeseo, descripcion, importe, desdeFecha, hastaFecha, idUsuario);

        return idDeseo;
    }


    public void altaDeseoFB(Integer idDeseo, String descripcion, float importe, String desdeFecha, String hastaFecha, String idUsuario) {

        objBD = database.getReference("Deseo");

        T_Deseo.Deseo nDeseo = new T_Deseo.Deseo(idDeseo, descripcion, importe, desdeFecha, hastaFecha, idUsuario);
        String item = objBD.child(idDeseo.toString()).getKey();
        objBD.child(item).setValue(nDeseo);
    }

    public void modificaDeseo(Integer idDeseo, String descripcion, float importe, String desdeFecha, String hastaFecha, String idUsuario) {

        T_Deseo.modificaDeseo(getApplicationContext(), idDeseo, descripcion, importe, desdeFecha, hastaFecha, idUsuario);

        if (conexOnline)
            modificaDeseoFB(idDeseo, descripcion, importe, desdeFecha, hastaFecha, idUsuario);

    }


    public void modificaDeseoFB(Integer idDeseo, String descripcion, float importe, String desdeFecha, String hastaFecha, String idUsuario) {
        objBD = database.getReference("Deseo");

        Map<String, T_Deseo.Deseo> nDeseo = new HashMap<>();
        nDeseo.put(idDeseo.toString(), new T_Deseo.Deseo(idDeseo, descripcion, importe, desdeFecha, hastaFecha, idUsuario));

        objBD.setValue(nDeseo);
    }


    public void bajaDeseo(Integer idDeseo) {

        //Se elimina de la base de datos Local y tambien se elimina de FB
        T_Deseo.eliminarDeseo(getApplicationContext(), idDeseo);

        //Se elimina el deseo de FB también
        bajaDeseoFB(idDeseo);

    }


    public void bajaDeseoFB(Integer idDeseo) {
        //Eliminar de la BD de FB el deseo indicado
        objBD = database.getReference("Deseo");


        objBD.child(idDeseo.toString()).removeValue();

    }



    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO PRESUPUESTO---------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */


    public Integer altaPresupuesto(String descripcion, float importe, String desdeFecha, String hastaFecha, String idUsuario) {
        Integer idPresupuesto;

        idPresupuesto = T_Presupuesto.altaPresupuesto(getApplicationContext(), descripcion, importe, desdeFecha, hastaFecha, idUsuario);

        if (conexOnline)
            altaPresupuestoFB(idPresupuesto, descripcion, importe, desdeFecha, hastaFecha, idUsuario);

        return idPresupuesto;
    }


    public void altaPresupuestoFB(Integer idPresupuesto, String descripcion, float importe, String desdeFecha, String hastaFecha, String idUsuario) {

        objBD = database.getReference("Presupuesto");

        T_Presupuesto.Presupuesto nPresupuesto = new T_Presupuesto.Presupuesto(idPresupuesto, descripcion, importe, desdeFecha, hastaFecha, idUsuario);
        String item = objBD.child(idPresupuesto.toString()).getKey();
        objBD.child(item).setValue(nPresupuesto);
    }

    public void modificaPresupuesto(Integer idPresupuesto, String descripcion, float importe, String desdeFecha, String hastaFecha, String idUsuario) {

        T_Presupuesto.modificaPresupuesto(getApplicationContext(), idPresupuesto, descripcion, importe, desdeFecha, hastaFecha, idUsuario);

        if (conexOnline)
            modificaPresupuestoFB(idPresupuesto, descripcion, importe, desdeFecha, hastaFecha, idUsuario);

    }


    public void modificaPresupuestoFB(Integer idPresupuesto, String descripcion, float importe, String desdeFecha, String hastaFecha, String idUsuario) {
        objBD = database.getReference("Presupuesto");

        Map<String, T_Presupuesto.Presupuesto> nPresupuesto = new HashMap<>();
        nPresupuesto.put(idPresupuesto.toString(), new T_Presupuesto.Presupuesto(idPresupuesto, descripcion, importe, desdeFecha, hastaFecha, idUsuario));

        objBD.setValue(nPresupuesto);
    }


    public void bajaPresupuesto(Integer idPresupuesto) {

        //Se elimina de la base de datos Local y tambien se elimina de FB
        T_Presupuesto.eliminarPresupuesto(getApplicationContext(), idPresupuesto);

        //Se elimina el presupuesto de FB también
        bajaPresupuestoFB(idPresupuesto);

    }


    public void bajaPresupuestoFB(Integer idPresupuesto) {
        //Eliminar de la BD de FB el presupuesto indicado
        objBD = database.getReference("Presupuesto");


        objBD.child(idPresupuesto.toString()).removeValue();

    }





    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO SITUACION-----------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */


    public Integer altaSituacion(String descripcion, float relacionGastoIngreso, String idUsuario) {

        Integer idSituacion;

        idSituacion = T_Situacion.altaSituacion(getApplicationContext(), descripcion, relacionGastoIngreso, idUsuario);

        if (conexOnline)
            altaSituacionFB(idSituacion, descripcion, relacionGastoIngreso, idUsuario);

        return idSituacion;
    }


    public void altaSituacionFB(Integer idSituacion, String descripcion,Float relacionGastoIngreso, String idUsuario) {

        objBD = database.getReference("Situacion");

        T_Situacion.Situacion nSituacion = new T_Situacion.Situacion(idSituacion, descripcion, relacionGastoIngreso, idUsuario);
        String item = objBD.child(idSituacion.toString()).getKey();
        objBD.child(item).setValue(nSituacion);
    }

    public void modificaSituacion(Integer idSituacion, String descripcion, float relacionGastoIngreso, String idUsuario) {

        T_Situacion.modificaSituacion(getApplicationContext(), idSituacion, descripcion, relacionGastoIngreso, idUsuario);

        if (conexOnline)
            modificaSituacionFB(idSituacion, descripcion, relacionGastoIngreso, idUsuario);

    }


    public void modificaSituacionFB(Integer idSituacion, String descripcion, float importe, String relacionGastoIngreso, String idUsuario) {
        objBD = database.getReference("Situacion");

        Map<String, T_Situacion.Situacion> nSituacion = new HashMap<>();
        nSituacion.put(idSituacion.toString(), new T_Situacion.Situacion(idSituacion, descripcion, relacionGastoIngreso, idUsuario));

        objBD.setValue(nPresupuesto);
    }


    public void bajaPresupuesto(Integer idPresupuesto) {

        //Se elimina de la base de datos Local y tambien se elimina de FB
        T_Presupuesto.eliminarPresupuesto(getApplicationContext(), idPresupuesto);

        //Se elimina el presupuesto de FB también
        bajaPresupuestoFB(idPresupuesto);

    }


    public void bajaPresupuestoFB(Integer idPresupuesto) {
        //Eliminar de la BD de FB el presupuesto indicado
        objBD = database.getReference("Presupuesto");


        objBD.child(idPresupuesto.toString()).removeValue();

    }


    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO PERIODICIDAD-----------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */

    public void bajaPeriodicidadFB(Integer idPeriodicidad) {
        //Eliminar de la BD de FB la Periodicidad indicada
        objBD = database.getReference("Periodicidad");

        Query query = objBD.orderByChild("idPeriodicidad").equalTo(idPeriodicidad);

        query.addValueEventListener(new ValueEventListener() {
            @Override

            public void onDataChange(DataSnapshot dataSnapshot) {

                DataSnapshot nPeriodicidad = dataSnapshot.getChildren().iterator().next();

                if (nPeriodicidad != null) {
                    String key = nPeriodicidad.getKey();
                    objBD.child(key).removeValue();

                }
            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


    }

    public void altaPeriodicidadFB(Integer idPeriodicidad, String descripcion) {
        objBD = database.getReference("Periodicidad");


        objBD.push().setValue(new T_Periodicidad.Tipo_Periodicidad(idPeriodicidad, descripcion));

    }


    public void modificaPeriodicidadFB(Integer idPeriodicdad, String descripcion) {
        objBD = database.getReference("Periodicidad");

        Map<String, T_Periodicidad.Tipo_Periodicidad> nPeriodicidad = new HashMap<>();
        nPeriodicidad.put(idPeriodicdad.toString(), new T_Periodicidad.Tipo_Periodicidad(idPeriodicdad, descripcion));

        objBD.setValue(nPeriodicidad);
    }




    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO USUARIO-------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */


    //Carga desplegable de Alimentacion
    public ArrayAdapter<String> cargaAlimentacion() {
        ArrayAdapter<String> dataAdapter = null;

        //Si no hay conexión se obtiene de BD local
        ArrayList<String> alimentaciones = new ArrayList<String>();
        alimentaciones.add(getResources().getString(R.string.ini_spinner));

        List<T_tipo_Alimentacion.Tipo_Alimentacion> alimentaciones_bd = T_tipo_Alimentacion.obtenerTodosTiposAlimentacion();

        for (int indice = 0; indice < alimentaciones_bd.size(); indice++) {
            alimentaciones.add(alimentaciones_bd.get(indice).getdescripcion());
        }

        dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, alimentaciones);

        return dataAdapter;
    }


    public Integer altaUsuario(String idUsuario, String pwd, String nombreCompleto, String email, Integer idAlimentacion, String alimentacion, String notificarEmail) {
        Integer codUsuario;

        codUsuario = T_Usuario.altaUsuario(getApplicationContext(), idUsuario, pwd, nombreCompleto, email, idAlimentacion, alimentacion, notificarEmail);

        if (conexOnline)
            altaUsuarioFB(codUsuario, idUsuario, pwd, nombreCompleto, email, idAlimentacion, alimentacion, notificarEmail);

        return codUsuario;
    }


    public void altaUsuarioFB(Integer codUsuario, String idUsuario, String pwd, String nombreCompleto, String email, Integer idAlimentacion, String alimentacion, String notificarEmail) {

        objBD = database.getReference("Usuario");

        T_Usuario.Usuario nUsuario = new T_Usuario.Usuario(codUsuario, idUsuario, pwd, nombreCompleto, email, idAlimentacion, alimentacion, notificarEmail);
        String item = objBD.child(codUsuario.toString()).getKey();
        objBD.child(item).setValue(nUsuario);
    }

    public void modificaUsuario(Integer codUsuario, String idUsuario, String pwd, String nombreCompleto, String email, Integer idAlimentacion, String alimentacion, String notificarEmail) {

        T_Usuario.modificaUsuario(getApplicationContext(), codUsuario, idUsuario, pwd, nombreCompleto, email, idAlimentacion, alimentacion, notificarEmail);

        if (conexOnline)
            modificaUsuarioFB(codUsuario, idUsuario, pwd, nombreCompleto, email, idAlimentacion, alimentacion, notificarEmail);

    }


    public void modificaUsuarioFB(Integer codUsuario, String idUsuario, String pwd, String nombreCompleto, String email, Integer idAlimentacion, String alimentacion, String notificarEmail) {
        objBD = database.getReference("Usuario");

        Map<String, T_Usuario.Usuario> nUsuario = new HashMap<>();
        nUsuario.put(codUsuario.toString(), new T_Usuario.Usuario(codUsuario, idUsuario, pwd, nombreCompleto, email, idAlimentacion, alimentacion, notificarEmail));

        objBD.setValue(nUsuario);
    }


    public void bajaUsuario(Integer codUsuario) {

        //Se elimina de la base de datos Local y tambien se elimina de FB
        T_Usuario.eliminarUsuario(getApplicationContext(), codUsuario);

        //Se elimina el usuario de FB también
        bajaUsuarioFB(codUsuario);

    }


    public void bajaUsuarioFB(Integer codUsuario) {
        //Eliminar de la BD de FB el deseo indicado
        objBD = database.getReference("Usuario");


        objBD.child(codUsuario.toString()).removeValue();

    }
}