package edu.uoc.android.gfp.modeloDatos;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteStatement;
import android.os.Build;
import android.support.annotation.RequiresApi;

import com.orm.SugarRecord;
import com.orm.query.Condition;
import com.orm.query.Select;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.android.gfp.utilidades.UtilidadesFecha;

public class T_Contribucion {

    public static final List<Contribucion> CONTRIBUCIONES = new ArrayList<>();

    //Procedimiento para obtener el total de Contribuciones en la BD local asociados al usuario
    //que ha hecho Login
    public static List<Contribucion> obtenerTodosContribuciones(String idUsuario) {

        List<Contribucion> contribuciones = Contribucion.find(Contribucion.class, "id_Usuario = ?", idUsuario);
        return contribuciones;

    }

    //Procedimiento para comprobar si Contribucion ya existe en la BD local
    public static boolean existe(Contribucion tContribucion) {

        List<Contribucion> mcontribucion = Contribucion.find(Contribucion.class, " id_Contribucion = ?", tContribucion.getIdContribucion().toString());

        if (mcontribucion == null || mcontribucion.size() == 0) {
            return false;
        } else {
            return true;
        }

    }

    //Procedimiento para obtener el último idContribucion
    public static Integer ultimoIdContribucion() {

        List<Contribucion> ultimoContribucion = Select.from(Contribucion.class)
                .orderBy("id_Contribucion")
                .list();

        if (ultimoContribucion.size() > 0)
            return ultimoContribucion.get(0).getIdContribucion();
        else
            return 0;

    }

    public static Contribucion leerContribucion(Integer idContribucion,String idUsuario){

        List<Contribucion> mContribucions = Select.from(Contribucion.class)
                .where(Condition.prop("id_Usuario").eq(idUsuario),
                        Condition.prop("id_Contribucion").eq(idContribucion))
                .list();


        return mContribucions.get(0);

    }


    public static float obtenerTotalContribucionesMensual (Context contexto, String idUsuario, Integer mes, Integer anyo){


        float totalContribuciones=0;



        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);
        SQLiteStatement query = db.compileStatement("SELECT SUM(importe) FROM Contribucion WHERE id_Usuario='" + idUsuario + "' and strftime('%m', fecha)='" + mes + "' and strftime('%Y', fecha)='" + anyo + "'");
        try {
            totalContribuciones = query.simpleQueryForLong();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            query.close();
        }

        return totalContribuciones;
    }


    public static void eliminarContribucion(Context contexto, Integer idContribucion){

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        SQLiteStatement query = db.compileStatement("delete from Contribucion where id_Contribucion=" + idContribucion);
        query.execute();
    }

    @RequiresApi(api = Build.VERSION_CODES.N)

    public static Integer altaContribucion(Context contexto, String descripcion, Integer idDeseo, float importe, String fecha,  String idUsuario,
                                      String deseo){

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        long idContribucion;

        ContentValues contentValues = new ContentValues();
        contentValues.put("descripcion", descripcion);
        contentValues.put("id_Deseo", idDeseo);
        contentValues.put("importe", importe);
        contentValues.put("fecha", fecha);
        contentValues.put("id_Usuario", idUsuario);
        contentValues.put("deseo", deseo);

        idContribucion=db.insert("contribucion","descripcion,  idDeseo,  importe,  fecha, id_Usuario,deseo",contentValues);

        SQLiteStatement query = db.compileStatement("update Contribucion set id_Contribucion=" + idContribucion + " where id=" + idContribucion);
        query.execute();

        return Math.toIntExact(idContribucion);

    }

    public static void modificaContribucion (Context contexto,Integer idContribucion, String descripcion, Integer idDeseo, float importe, String fecha, String idUsuario,
                                        String deseo){

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        ContentValues contentValues = new ContentValues();
        contentValues.put("descripcion", descripcion);
        contentValues.put("id_Deseo", idDeseo);
        contentValues.put("importe", importe);
        contentValues.put("fecha", fecha);
        contentValues.put("id_Usuario", idUsuario);
        contentValues.put("deseo", deseo);

        db.update("Contribucion",contentValues,"id_Contribucion = ? ", new String[]{idContribucion.toString()});

    }


    public static List<T_Contribucion.Contribucion> obtenerDetalleContribuciones (Context contexto, String descripcion, String dimporte, String himporte, String dfecha, String hfecha, String idUsuario,
                                                                  String deseo)
    {

        List<T_Contribucion.Contribucion> mcontribucion= new ArrayList<>();
        T_Contribucion.Contribucion contribucion;

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        String consulta;
        String condicion="";
        String separador;


        separador="";

        if (!descripcion.isEmpty())
        {
            condicion= " descipcion ='" + descripcion + "' ";
            separador=" and ";
        }


        if (!dimporte.isEmpty())
        {
            condicion= condicion + separador + " importe >= " + dimporte;
            separador=" and ";
        }


        if (!himporte.isEmpty())
        {
            condicion= condicion + separador + " importe <= " + himporte;
            separador=" and ";
        }

        if (!dfecha.isEmpty())
        {
            condicion= condicion + separador + " fecha >= " + UtilidadesFecha.formatearFecha(dfecha);
            separador=" and ";
        }

        if (!hfecha.isEmpty())
        {
            condicion= condicion + separador + " fecha <= " + UtilidadesFecha.formatearFecha(hfecha);
            separador=" and ";
        }

        if (!idUsuario.isEmpty())
        {
            condicion= condicion + separador + " id_Usuario = '" + idUsuario +"'";
            separador=" and ";
        }

        if (!deseo.isEmpty())
        {
            condicion= condicion + separador + " deseo = '" + deseo +"'";
            separador=" and ";
        }


        consulta=" select * from contribucion where " + condicion;

        Cursor c = db.rawQuery(consulta,null);

        c.moveToFirst();

        while (!c.isAfterLast())
        {

            // 0 ID INTEGER PRIMARY KEY AUTOINCREMENT ,
            // 1 DESCRIPCION TEXT,
            // 2 DESEO TEXT,
            // 3 FECHA TEXT,
            // 4 ID_CONTRIBUCION INTEGER,
            // 5 ID_DESEO INTEGER,
            // 6 ID_USUARIO TEXT,
            // 7 IMPORTE FLOAT

            contribucion = new T_Contribucion.Contribucion();
            contribucion.idContribucion=Integer.parseInt(c.getString(4));
            contribucion.descripcion=c.getString(1);
            contribucion.importe=Float.parseFloat(c.getString(7));
            contribucion.fecha=UtilidadesFecha.formatearFecha(c.getString(3));
            contribucion.deseo=c.getString(2);
            contribucion.idUsuario=c.getString(6);

            mcontribucion.add(contribucion); //add the item
            c.moveToNext();
        }


        return mcontribucion;
    }


    public static class Contribucion  extends  SugarRecord{


        public Integer idContribucion;
        public String descripcion;
        public Float importe;
        public Integer idDeseo;
        public String fecha;
        public String idUsuario;
        //se añade este campo que aunque genera redundancia facilitará la gestión de los registros
        //de cara a mostrar toda la info y no tener que utilizar vistas/
        //Como mejora app utilizar como SGBD SQL
        public String deseo;


        public Contribucion()
        {
            // Default constructor required for calls to DataSnapshot.getValue(Contribucion.class)
        }

        public  Contribucion(Integer idContribucion,String descripcion,Float importe,Integer idDeseo, String fecha, String idUsuario,String deseo)
        {
            this.idContribucion=idContribucion;
            this.descripcion=descripcion;
            this.importe=importe;
            this.idDeseo=idDeseo;
            this.fecha=fecha;
            this.idUsuario=idUsuario;
            this.deseo=deseo;
        }


        public Integer getIdContribucion() {
            return idContribucion;
        }

        public void setIdContribucion(Integer idContribucion) {
            this.idContribucion = idContribucion;
        }

        public String getdescripcion() {
            return descripcion;
        }

        public void setdescripcion(String descripcion) {
            this.descripcion = descripcion;
        }

        public Float getimporte() {
            return importe;
        }

        public void setimporte(float importe) {
            this.importe = importe;
        }

        public Integer getIdDeseo() {
            return idDeseo;
        }

        public void setIdDeseo(Integer idDeseo) {
            this.idDeseo = idDeseo;
        }

        public String getFecha() {
            return fecha;
        }

        public void setFecha(String fecha) {
            this.fecha = fecha;
        }


        public String getidUsuario() {
            return idUsuario;
        }

        public void setidUsuario(String idUsuario) {
            this.idUsuario = idUsuario;
        }

        public String getDeseo() {
            return deseo;
        }

        public void setDeseo(String deseo) {
            this.deseo = deseo;
        }

        @Override
        public String toString() {

            return "Contribuacion {" +
                    "idContribucion=" + idContribucion +
                    "descripcion='" + descripcion + '\'' +
                    "importe=" + importe +
                    "fecha='" + fecha + '\'' +
                    "idDeseo=" + idDeseo +
                    "idUsuario='" + idUsuario + '\'' +
                    ", descripcion=" + descripcion + '\'' +
                    ", deseo=" + deseo + '\'' +
                    '}';
        }



    }







}