package edu.uoc.android.gfp.modeloDatos;


import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteStatement;
import android.os.Build;
import android.support.annotation.RequiresApi;

import com.orm.SugarRecord;
import com.orm.query.Condition;
import com.orm.query.Select;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.android.gfp.utilidades.UtilidadesFecha;

public class T_Gasto {

    public static final List<Gasto> GASTOS = new ArrayList<>();

    //Procedimiento para obtener el total de gastos en la BD local asociados al usuario
    //que ha hecho Login
    public static List<Gasto> obtenerTodosGasto(String idUsuario) {

        List<Gasto> gastos = Gasto.find(Gasto.class, "id_Usuario = ?", idUsuario);
        return gastos;

    }

    //Procedimiento para comprobar si Gasto ya existe en la BD local
    public static boolean existe(Gasto tgasto) {

        List<Gasto> tgasto1 = Gasto.find(Gasto.class, "id_Gasto = ?", String.valueOf(tgasto.idGasto));

        if (tgasto1 == null || tgasto1.size() == 0) {
            return false;
        } else {
            return true;
        }

    }

    //Procedimiento para obtener el último idTipoGasto
    public static Integer ultimoIdGasto() {

        List<Gasto> ultimoGasto = Select.from(Gasto.class)
                .orderBy("id_Gasto")
                .list();

        if (ultimoGasto.size() > 0)
            return ultimoGasto.get(0).getIdGasto();
        else
            return 0;

    }

    public static Gasto leerGasto(Integer idGasto,String idUsuario){

        List<Gasto> mgastos = Select.from(Gasto.class)
                .where(Condition.prop("id_Usuario").eq(idUsuario),
                        Condition.prop("id_Gasto").eq(idGasto))
                .list();

        mgastos.size();

        return mgastos.get(0);

    }

    public static float obtenerTotalGastosMensual (Context contexto,String idUsuario, Integer mes,Integer anyo){


       float totalGastos=0;



        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        SQLiteStatement query = db.compileStatement("SELECT sum(importe)  FROM Gasto WHERE id_Usuario='" + idUsuario + "' and strftime('%M', fecha_Gasto)='" + mes + "' and strftime('%Y', fecha_Gasto)='" + anyo + "'");
        try {
            totalGastos = query.simpleQueryForLong();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            query.close();
        }

        return totalGastos;
    }

    public static void eliminarGasto(Context contexto, Integer idGasto){

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        SQLiteStatement query = db.compileStatement("delete from Gasto where id_Gasto=" + idGasto);
        query.execute();
    }

    @RequiresApi(api = Build.VERSION_CODES.N)
    public static Integer altaGasto(Context contexto, String descripcion, Integer idTipoGasto, float importe, String fechaGasto, Integer idPeriodicidad, Integer idNecesidad, String idUsuario,
                                    String tipoGasto, String periodicidad, String necesidad){

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        long idGasto;

        //SQLiteStatement query = db.compileStatement("insert into Gasto (descripcion,  id_Tipo_Gasto,  importe,  fecha_Gasto,  id_Periodicidad,  id_Necesidad,  id_Usuario,tipo_Gasto,periodicidad,necesidad) values ('" + descripcion +"'," +  idTipoGasto + "," +  importe + ",'"  + fechaGasto + "'," +  idPeriodicidad + "," + idNecesidad + ",'" +  idUsuario +"','" + tipoGasto + "','" + periodicidad +"','" + necesidad +"')");
        //query.execute();

        ContentValues contentValues = new ContentValues();
        contentValues.put("descripcion", descripcion);
        contentValues.put("id_Tipo_Gasto", idTipoGasto);
        contentValues.put("importe", importe);
        contentValues.put("fecha_Gasto", fechaGasto);
        contentValues.put("id_Periodicidad", idPeriodicidad);
        contentValues.put("id_Necesidad", idNecesidad);
        contentValues.put("id_Usuario", idUsuario);
        contentValues.put("tipo_Gasto", tipoGasto);
        contentValues.put("periodicidad", periodicidad);
        contentValues.put("necesidad", necesidad);

        idGasto=db.insert("Gasto","descripcion,  id_Tipo_Gasto,  importe,  fecha_Gasto,  id_Periodicidad,  id_Necesidad,  id_Usuario,tipo_Gasto,periodicidad,necesidad",contentValues);

        SQLiteStatement query = db.compileStatement("update Gasto set id_Gasto=" + idGasto + " where id=" + idGasto);
        query.execute();

        return Math.toIntExact(idGasto);

    }

    public static void modificaGasto (Context contexto,Integer idGasto, String descripcion, Integer idTipoGasto, float importe, String fechaGasto, Integer idPeriodicidad, Integer idNecesidad, String idUsuario,
                                    String tipoGasto, String periodicidad, String necesidad){

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        ContentValues contentValues = new ContentValues();
        contentValues.put("descripcion", descripcion);
        contentValues.put("id_Tipo_Gasto", idTipoGasto);
        contentValues.put("importe", importe);
        contentValues.put("fecha_Gasto", fechaGasto);
        contentValues.put("id_Periodicidad", idPeriodicidad);
        contentValues.put("id_Necesidad", idNecesidad);
        contentValues.put("id_Usuario", idUsuario);
        contentValues.put("tipo_Gasto", tipoGasto);
        contentValues.put("periodicidad", periodicidad);
        contentValues.put("necesidad", necesidad);

        db.update("Gasto",contentValues,"id_gasto = ? ", new String[]{idGasto.toString()});

    }


    public static List<Gasto> obtenerDetalleGastos (Context contexto, String descripcion, String dimporte, String himporte, String dfechaGasto, String hfechaGasto, String idUsuario,
                                                    String tipoGasto, String periodicidad)
        {

            List<Gasto> mgastos= new ArrayList<>();
            Gasto gasto;

            SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        String consulta;
        String condicion="";
        String separador;


        separador="";

        if (!descripcion.isEmpty())
        {
            condicion= " descipcion ='" + descripcion + "' ";
            separador=" and ";
        }


        if (!dimporte.isEmpty())
        {
            condicion= condicion + separador + " importe >= " + dimporte;
            separador=" and ";
        }


        if (!himporte.isEmpty())
        {
            condicion= condicion + separador + " importe <= " + himporte;
            separador=" and ";
        }

        if (!dfechaGasto.isEmpty())
        {
            condicion= condicion + separador + " fecha_Gasto >= " + UtilidadesFecha.formatearFecha(dfechaGasto);
            separador=" and ";
        }

        if (!hfechaGasto.isEmpty())
        {
            condicion= condicion + separador + " fecha_Gasto <= " + UtilidadesFecha.formatearFecha(hfechaGasto);
            separador=" and ";
        }

        if (!idUsuario.isEmpty())
        {
            condicion= condicion + separador + " id_Usuario = '" + idUsuario +"'";
            separador=" and ";
        }

        if (!tipoGasto.isEmpty())
        {
            condicion= condicion + separador + " tipo_Gasto = '" + tipoGasto +"'";
            separador=" and ";
        }

        if (!periodicidad.isEmpty())
        {
            condicion= condicion + separador + " periodicidad ='" + periodicidad + "'";
            separador=" and ";
        }

        consulta=" select * from Gasto where " + condicion;

        Cursor c = db.rawQuery(consulta,null);

        c.moveToFirst();

        while (!c.isAfterLast())
        {

            // 0 ID INTEGER PRIMARY KEY AUTOINCREMENT
            // 1 DESCRIPCION TEXT,
            // 2 FECHA_GASTO TEXT,
            // 3 ID_GASTO INTEGER,
            // 4 ID_NECESIDAD INTEGER,
            // 5 ID_PERIODICIDAD INTEGER,
            // 6 ID_TIPO_GASTO INTEGER,
            // 7 ID_USUARIO TEXT,
            // 8 IMPORTE FLOAT,
            // 9 NECESIDAD TEXT,
            // 10 PERIODICIDAD TEXT,
            // 11 TIPO_GASTO TEXT

            gasto = new Gasto();
            gasto.idGasto=Integer.parseInt(c.getString(3));
            gasto.descripcion=c.getString(1);
            gasto.importe=Float.parseFloat(c.getString(8));
            gasto.fechaGasto=UtilidadesFecha.formatearFecha(c.getString(2));
            gasto.tipoGasto=c.getString(11);
            gasto.periodicidad =c.getString(10);
            gasto.idUsuario=c.getString(7);

            mgastos.add(gasto); //add the item
            c.moveToNext();
        }


        return mgastos;
    }

    public static class Gasto extends SugarRecord {

        public Integer idGasto;
        public String descripcion;
        public Integer idTipoGasto;
        public float importe;
        public String fechaGasto;
        public Integer idPeriodicidad;
        public Integer idNecesidad;
        public String idUsuario;

        //se añaden estos campos que aunque genera redundancia facilitará la gestión de los registros
        //de cara a mostrar toda la info y no tener que utilizar vistas/
        //Como mejora app utilizar como SGBD SQL
        public String tipoGasto;
        public String periodicidad;
        public String necesidad;


        public Gasto() {
            // Default constructor required for calls to DataSnapshot.getValue(User.class)
        }

        public Gasto(Integer idGasto, String descripcion, Integer idTipoGasto, float importe, String fechaGasto,
                     Integer idPeriodicidad, Integer idNecesidad, String idUsuario,String tipoGasto,String periodicidad,
                     String necesidad) {

            this.idGasto = idGasto;
            this.descripcion = descripcion;
            this.idTipoGasto = idTipoGasto;
            this.importe = importe;
            this.fechaGasto = fechaGasto;
            this.idPeriodicidad = idPeriodicidad;
            this.idNecesidad = idNecesidad;
            this.idUsuario = idUsuario;
            this.tipoGasto=tipoGasto;
            this.periodicidad=periodicidad;
            this.necesidad=necesidad;

        }

        public Integer getIdGasto() {
            return idGasto;
        }

        public void setIdGasto(Integer idGasto) {
            this.idGasto = idGasto;
        }

        public String getdescripcion() {
            return descripcion;
        }

        public void setdescripcion(String descripcion) {
            this.descripcion = descripcion;
        }

        public Integer getidTipoGasto() {
            return idTipoGasto;
        }

        public void setidTipoGasto(Integer idTipoGasto) {
            this.idTipoGasto = idTipoGasto;
        }

        public Float getimporte() {
            return importe;
        }

        public void setimporte(float importe) {
            this.importe = importe;
        }

        public String getfechaGasto() {
            return fechaGasto;
        }

        public void setFechaGasto(String fechaGasto) {
            this.fechaGasto = fechaGasto;
        }

        public Integer getidPeriodicidad() {
            return idPeriodicidad;
        }

        public void setidPeriodicidad(Integer idPeriodicidad) {
            this.idPeriodicidad = idPeriodicidad;
        }

        public Integer getidNecesidad() {
            return idNecesidad;
        }

        public void setidNecesidad(Integer idNecesidad) {
            this.idNecesidad = idNecesidad;
        }

        public String getTipoGasto() {
            return tipoGasto;
        }

        public void setTipoGasto(String tipoGasto) {
            this.tipoGasto = tipoGasto;
        }

        public String getPeriodicidad() {
            return periodicidad;
        }

        public void setPeriodicidad(String periodicidad) {
            this.periodicidad = periodicidad;
        }

        public String getNecesidad() {
            return necesidad;
        }

        public void setNecesidad(String necesidad) {
            this.necesidad = necesidad;
        }

        public String getIdUsuario() {
            return idUsuario;
        }

        public void setidUsuario(String idUsuario) {
            this.idUsuario = idUsuario;
        }

        @Override
        public String toString() {

            return "Gasto {" +
                    "idGasto=" + idGasto +
                    "descripcion='" + descripcion + '\'' +
                    "importe=" + importe +
                    "fechaGasto='" + fechaGasto + '\'' +
                    "idPeriodicidad=" + idPeriodicidad +
                    "idNecesidad=" + idNecesidad +
                    "idUsuario='" + idUsuario + '\'' +
                    "idTipoGasto='" + idTipoGasto +
                    ", descripcion=" + descripcion + '\'' +
                    ", idNecesidad='" + idNecesidad +
                    '}';
        }

    }

}