package edu.uoc.android.gfp.modeloDatos;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteStatement;
import android.os.Build;
import android.support.annotation.RequiresApi;

import com.orm.SugarRecord;
import com.orm.query.Condition;
import com.orm.query.Select;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.android.gfp.utilidades.UtilidadesFecha;

public class T_Ingreso {

    public static final List<Ingreso> INGRESOS = new ArrayList<>();

    //Procedimiento para obtener el total de gastos en la BD local asociados al usuario
    //que ha hecho Login
    public static List<Ingreso> obtenerTodosIngreso(String idUsuario) {

        List<Ingreso> ingresos = Ingreso.find(Ingreso.class, "id_Usuario = ?", idUsuario);
        return ingresos;

    }

    //Procedimiento para comprobar si Gasto ya existe en la BD local
    public static boolean existe(Ingreso tIngreso) {

        List<Ingreso> ingreso = Ingreso.find(Ingreso.class, " idIngreso = ?", String.valueOf(tIngreso.idIngreso));

        if (ingreso == null || ingreso.size() == 0) {
            return false;
        } else {
            return true;
        }

    }

    //Procedimiento para obtener el último idTipoGasto
    public static Integer ultimoIdIngreso() {

        List<Ingreso> ultimoIngreso = Select.from(Ingreso.class)
                .orderBy("id_Ingreso")
                .list();

        if (ultimoIngreso.size() > 0)
            return ultimoIngreso.get(0).getIdIngreso();
        else
            return 0;

    }

    public static Ingreso leerIngreso(Integer idIngreso,String idUsuario){

        List<Ingreso> mIngresos = Select.from(Ingreso.class)
                .where(Condition.prop("id_Usuario").eq(idUsuario),
                        Condition.prop("id_Ingreso").eq(idIngreso))
                .list();


        return mIngresos.get(0);

    }

    public static float obtenerTotalIngresosMensual (Context contexto, String idUsuario, Integer mes, Integer anyo){


        float totalIngresos=0;



        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);
        SQLiteStatement query = db.compileStatement("SELECT SUM(importe) FROM Ingreso WHERE id_Usuario='" + idUsuario + "' and strftime('%m', fecha_Ingreso)='" + mes + "' and strftime('%Y', fecha_Ingreso)='" + anyo + "'");
        try {
            totalIngresos = query.simpleQueryForLong();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            query.close();
        }

        return totalIngresos;
    }


    public static void eliminarIngreso(Context contexto, Integer idIngreso){

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        SQLiteStatement query = db.compileStatement("delete from Ingreso where id_Ingreso=" + idIngreso);
        query.execute();
    }

    @RequiresApi(api = Build.VERSION_CODES.N)

    public static Integer altaIngreso(Context contexto, String descripcion, Integer idTipoIngreso, float importe, String fechaIngreso, Integer idPeriodicidad, String idUsuario,
                                    String tipoIngreso, String periodicidad){

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        long idIngreso;

        ContentValues contentValues = new ContentValues();
        contentValues.put("descripcion", descripcion);
        contentValues.put("id_Tipo_Ingreso", idTipoIngreso);
        contentValues.put("importe", importe);
        contentValues.put("fecha_Ingreso", fechaIngreso);
        contentValues.put("id_Periodicidad", idPeriodicidad);
        contentValues.put("id_Usuario", idUsuario);
        contentValues.put("tipo_Ingreso", tipoIngreso);
        contentValues.put("periodicidad", periodicidad);


        idIngreso=db.insert("Ingreso","descripcion,  id_Tipo_Ingreso,  importe,  fecha_Ingreso,  id_Periodicidad, id_Usuario,tipo_Ingreso,periodicidad",contentValues);

        SQLiteStatement query = db.compileStatement("update Ingreso set id_Ingreso=" + idIngreso + " where id=" + idIngreso);
        query.execute();

        return Math.toIntExact(idIngreso);

    }

    public static void modificaIngreso (Context contexto,Integer idIngreso, String descripcion, Integer idTipoIngreso, float importe, String fechaIngreso, Integer idPeriodicidad, String idUsuario,
                                      String tipoIngreso, String periodicidad){

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        ContentValues contentValues = new ContentValues();
        contentValues.put("descripcion", descripcion);
        contentValues.put("id_Tipo_Ingreso", idTipoIngreso);
        contentValues.put("importe", importe);
        contentValues.put("fecha_ingreso", fechaIngreso);
        contentValues.put("id_Periodicidad", idPeriodicidad);
        contentValues.put("id_Usuario", idUsuario);
        contentValues.put("tipo_ingreso", tipoIngreso);
        contentValues.put("periodicidad", periodicidad);

        db.update("Ingreso",contentValues,"id_Ingreso = ? ", new String[]{idIngreso.toString()});

    }


    public static List<T_Ingreso.Ingreso> obtenerDetalleIngresos (Context contexto, String descripcion, String dimporte, String himporte, String dfechaIngreso, String hfechaIngreso, String idUsuario,
                                                            String tipoIngreso, String periodicidad)
    {

        List<T_Ingreso.Ingreso> mingresos= new ArrayList<>();
        T_Ingreso.Ingreso ingreso;

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        String consulta;
        String condicion="";
        String separador;


        separador="";

        if (!descripcion.isEmpty())
        {
            condicion= " descipcion ='" + descripcion + "' ";
            separador=" and ";
        }


        if (!dimporte.isEmpty())
        {
            condicion= condicion + separador + " importe >= " + dimporte;
            separador=" and ";
        }


        if (!himporte.isEmpty())
        {
            condicion= condicion + separador + " importe <= " + himporte;
            separador=" and ";
        }

        if (!dfechaIngreso.isEmpty())
        {
            condicion= condicion + separador + " fecha_Ingreso >= " + UtilidadesFecha.formatearFecha(dfechaIngreso);
            separador=" and ";
        }

        if (!hfechaIngreso.isEmpty())
        {
            condicion= condicion + separador + " fecha_Ingreso <= " + UtilidadesFecha.formatearFecha(hfechaIngreso);
            separador=" and ";
        }

        if (!idUsuario.isEmpty())
        {
            condicion= condicion + separador + " id_Usuario = '" + idUsuario +"'";
            separador=" and ";
        }

        if (!tipoIngreso.isEmpty())
        {
            condicion= condicion + separador + " tipo_Ingreso = '" + tipoIngreso +"'";
            separador=" and ";
        }

        if (!periodicidad.isEmpty())
        {
            condicion= condicion + separador + " periodicidad ='" + periodicidad + "'";
            separador=" and ";
        }

        consulta=" select * from Ingreso where " + condicion;

        Cursor c = db.rawQuery(consulta,null);

        c.moveToFirst();

        while (!c.isAfterLast())
        {

            // 0 ID INTEGER PRIMARY KEY AUTOINCREMENT ,
            // 1 DESCRIPCION TEXT,
            // 2 FECHA_INGRESO TEXT,
            // 3 ID_INGRESO INTEGER,
            // 4 ID_PERIODICIDAD INTEGER,
            // 5 ID_TIPO_INGRESO INTEGER,
            // 6 ID_USUARIO TEXT,
            // 7 IMPORTE FLOAT,
            // 8 PERIODICIDAD TEXT,
            // 9 TIPO_INGRESO TEXT

            ingreso = new T_Ingreso.Ingreso();
            ingreso.idIngreso=Integer.parseInt(c.getString(3));
            ingreso.descripcion=c.getString(1);
            ingreso.importe=Float.parseFloat(c.getString(7));
            ingreso.fechaIngreso=UtilidadesFecha.formatearFecha(c.getString(2));
            ingreso.tipoIngreso=c.getString(9);
            ingreso.periodicidad =c.getString(8);
            ingreso.idUsuario=c.getString(6);

            mingresos.add(ingreso); //add the item
            c.moveToNext();
        }


        return mingresos;
    }




    public static class Ingreso extends SugarRecord{

        public Integer idIngreso;
        public String descripcion;
        public Integer idTipoIngreso;
        public float importe;
        public String fechaIngreso;
        public Integer idPeriodicidad;
        public String idUsuario;

        //se añaden estos campos que aunque genera redundancia facilitará la gestión de los registros
        //de cara a mostrar toda la info y no tener que utilizar vistas/
        //Como mejora app utilizar como SGBD SQL
        public String tipoIngreso;
        public String periodicidad;


        public Ingreso() {
            // Default constructor required for calls to DataSnapshot.getValue(User.class)
        }

        public Ingreso(Integer idIngreso,String descripcion,Integer idTipoIngreso,float importe,String fechaIngreso,Integer idPeriodicidad,String idUsuario,String tipoIngreso,String periodicidad) {

            this.idIngreso=idIngreso;
            this.descripcion=descripcion;
            this.idTipoIngreso=idTipoIngreso;
            this.importe=importe;
            this.fechaIngreso=fechaIngreso;
            this.idPeriodicidad=idPeriodicidad;
            this.idUsuario=idUsuario;
            this.tipoIngreso=tipoIngreso;
            this.periodicidad=periodicidad;

        }

        public Integer getIdIngreso() {
            return idIngreso;
        }

        public void setIdIngreso(Integer idIngreso) {
            this.idIngreso = idIngreso;
        }

        public String getdescripcion() {
            return descripcion;
        }

        public void setdescripcion(String descripcion) {
            this.descripcion = descripcion;
        }

        public Integer getIdTipoIngreso() {
            return idTipoIngreso;
        }

        public void setIdTipoIngreso(Integer idTipoIngreso) {
            this.idTipoIngreso = idTipoIngreso;
        }

        public Float getimporte() {
            return importe;
        }

        public void setimporte(float importe) {
            this.importe = importe;
        }

        public String getFechaIngreso() {
            return fechaIngreso;
        }

        public void setFechaIngreso(String fechaIngreso) {
            this.fechaIngreso = fechaIngreso;
        }

        public Integer getidPeriodicidad() {
            return idPeriodicidad;
        }

        public void setidPeriodicidad(Integer idPeriodicidad) {
            this.idPeriodicidad = idPeriodicidad;
        }

        public String getidUsuario() {
            return idUsuario;
        }

        public void setidUsuario(String idUsuario) {
            this.idUsuario = idUsuario;
        }

        public String getTipoIngreso() {
            return tipoIngreso;
        }

        public void setTipoIngreso(String tipoIngreso) {
            this.tipoIngreso = tipoIngreso;
        }

        public String getPeriodicidad() {
            return periodicidad;
        }

        public void setPeriodicidad(String periodicidad) {
            this.periodicidad = periodicidad;
        }

        @Override
        public String toString() {

            return "Ingreso {" +
                    "idIngreso=" + idIngreso +
                    "descripcion='" + descripcion + '\'' +
                    "importe=" + importe +
                    "fechaIngreso='" + fechaIngreso + '\'' +
                    "idPeriodicidad=" + idPeriodicidad +
                    "idUsuario='" + idUsuario + '\'' +
                    "idTipoIngreso='" + idTipoIngreso +
                    ", descripcion=" + descripcion + '\'' +
                    ", tipoIngreso=" + tipoIngreso + '\'' +
                    ", periodicidad=" + periodicidad + '\'' +
                    '}';
        }



    }







}