package edu.uoc.android.gfp.vistas.Gasto;

import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.view.Gravity;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;


import edu.uoc.android.gfp.MyApplication;
import edu.uoc.android.gfp.R;
import edu.uoc.android.gfp.utilidades.UtilidadesTeclado;
import edu.uoc.android.gfp.vistas.Deseo.RegistroDeseo;
import edu.uoc.android.gfp.vistas.FragmentoCabecera;
import edu.uoc.android.gfp.utilidades.UtilidadesFecha;


public class RegistroGasto extends AppCompatActivity implements View.OnClickListener {


    // Variables de los elementos de la pantalla
    private EditText txtDescripcion, txtImporte, txtFecha;

    private Button btnCancelar, btnGuardar;
    private Spinner despTipoGasto, despPeriodicidad;


    //Variables necesarias para registrar un Gasto
    public String descripcion;
    public Integer idTipoGasto=-1;
    public float importe;
    public String fechaGasto;
    public Integer idPeriodicidad=-1;
    public Integer idNecesidad=1;
    public String idUsuario;
    public String tipoGasto="";
    public String periodicidad="";
    public String necesidad="";
    public Integer idGasto;


    //Para la gestión de ocultar el teclado una vez que un campo de texto pierde el foco
    public UtilidadesTeclado utilTecla;
    public UtilidadesFecha utilFecha;


    public MyApplication myApplication;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        {

            super.onCreate(savedInstanceState);
            setContentView(R.layout.registrogasto);

            idUsuario= getIntent().getExtras().getString("idUsuario");

            if (savedInstanceState == null) {
                //Fragmento para incluir la cabecera
                Bundle arguments = new Bundle();
                arguments.putString(FragmentoCabecera.arg_idUsuario, idUsuario);
                arguments.putString(FragmentoCabecera.arg_funcionalidad, "Alta Nuevo Gasto");
                FragmentoCabecera fragment = new FragmentoCabecera();
                fragment.setArguments(arguments);
                getSupportFragmentManager().beginTransaction()
                        .add(R.id.frame_cabecera, fragment)
                        .commit();
            }


            //Para la gestión de ocultar el teclado una vez que un campo de texto pierde el foco
            utilTecla = new UtilidadesTeclado(this);
            utilFecha = new UtilidadesFecha();


            //Inicialización de todos los elementos de la pantalla

            txtDescripcion = (EditText) findViewById(R.id.t_descripcion);
            txtDescripcion.setOnFocusChangeListener(new View.OnFocusChangeListener() {
                @Override
                public void onFocusChange(View v, boolean hasFocus) {
                    if(!hasFocus)
                    {
                        utilTecla.hideKeyBoard(v);
                    }
                }
            });


            txtImporte = (EditText) findViewById(R.id.t_importe);
            txtImporte.setOnFocusChangeListener(new View.OnFocusChangeListener() {
                @Override
                public void onFocusChange(View v, boolean hasFocus) {
                    if(!hasFocus)
                    {
                        utilTecla.hideKeyBoard(v);
                    }
                }
            });

            txtFecha = (EditText) findViewById(R.id.t_fecha);
            txtFecha.setOnFocusChangeListener(new View.OnFocusChangeListener() {
                @Override
                public void onFocusChange(View v, boolean hasFocus) {
                    if(!hasFocus)
                    {
                        if (!txtFecha.getText().toString().isEmpty())
                        {
                            if (!utilFecha.validarFecha(txtFecha.getText().toString())) {
                                //La validación de la fecha no ha sido correcta
                                //mostramos mensaje
                                Toast.makeText(RegistroGasto.this, "Formato de fecha incorrecto mm/dd/yyyy", Toast.LENGTH_SHORT).show();

                            }
                        }
                        utilTecla.hideKeyBoard(v);
                    }
                }
            });

            btnCancelar = (Button) findViewById(R.id.b_cancelar);
            btnGuardar = (Button) findViewById(R.id.b_guardar);

            // Metodos Listener
            btnCancelar.setOnClickListener((View.OnClickListener) this);
            btnGuardar.setOnClickListener((View.OnClickListener) this);


            despTipoGasto = (Spinner) findViewById(R.id.t_tipogasto);
            despPeriodicidad = (Spinner) findViewById(R.id.t_periodicidad);

            myApplication = (MyApplication) getApplicationContext();


            // Carga desplegable de tipo Gasto
            despTipoGasto.setAdapter((ArrayAdapter<String>) myApplication.cargaTipoGasto());

            despTipoGasto.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {

                @Override
                public void onItemSelected(AdapterView<?> arg0, View arg1,
                                           int pos, long id) {

                    String descripcion = arg0.getItemAtPosition(pos)
                            .toString();

                    if (descripcion.equals(getResources().getString(R.string.ini_spinner))) {
                        //No se entiende seleccionado ningún valor
                        tipoGasto="";
                        idTipoGasto = -1;
                    } else {
                        tipoGasto=descripcion;
                        idTipoGasto = pos;


                    }

                }

                @Override
                public void onNothingSelected(AdapterView<?> arg0) {
                    // TODO Auto-generated method stub

                }
            });


            // Carga desplegable de tipo Periodicidad

            despPeriodicidad.setAdapter((ArrayAdapter<String>) myApplication.cargaTipoPeriodicidad());

            despPeriodicidad.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {

                @Override
                public void onItemSelected(AdapterView<?> arg0, View arg1,
                                           int pos, long id) {

                    String descripcion = arg0.getItemAtPosition(pos)
                            .toString();

                    if (descripcion.equals(getResources().getString(R.string.ini_spinner))) {
                        //No se entiende seleccionado ningún valor
                        idPeriodicidad = -1;
                        periodicidad="";
                    } else {
                        idPeriodicidad = pos;
                        periodicidad=descripcion;
                    }

                }

                @Override
                public void onNothingSelected(AdapterView<?> arg0) {
                    // TODO Auto-generated method stub

                }
            });

        }


    }


    @Override
    public void onClick (View v){

        Boolean continua;

        continua=true;

        // Dependiendo si se pulsa Login o Registro ejecuta una actividad u otra
        switch (v.getId()) {
            case R.id.b_guardar:

             if (txtDescripcion.toString().isEmpty())
                    continua=false;

                if (txtImporte.getText().toString().isEmpty())
                    continua=false;

                if (txtFecha.getText().toString().isEmpty())
                    continua=false;

                if (idTipoGasto==-1)
                    continua=false;

                if (idPeriodicidad==-1)
                    continua=false;

                if (idNecesidad==-1)
                    continua=false;

                if (continua==true)
                {


                    descripcion=txtDescripcion.getText().toString();
                    //el valor ya está asignado idTipoGasto
                    importe= Float.parseFloat(txtImporte.getText().toString());
                    fechaGasto=utilFecha.formatearFecha(txtFecha.getText().toString());

                    //el valor ya está asignado idPeriodicidad
                    //el valor ya está asignado idNecesidad;
                    //el valor ya está asignado idUsuario;


                    necesidad="ninguna";

                    //myApplication.altaGastoFB(idGasto, descripcion, idTipoGasto, importe, fechaGasto, idPeriodicidad, idNecesidad, idUsuario,tipoGasto,periodicidad,necesidad);
                    idGasto=myApplication.altaGasto(descripcion, idTipoGasto, importe, fechaGasto, idPeriodicidad, idNecesidad, idUsuario,tipoGasto,periodicidad,necesidad);

                    //Una vez se ha registadrado el gasto se redirige a la ficha del gasto
                    Intent intent_mg = new Intent(getApplicationContext(), FichaGasto.class);
                    intent_mg.putExtra("arg_idUsuario", idUsuario);
                    intent_mg.putExtra("arg_idGasto", idGasto);
                    startActivity(intent_mg);


                }
                else
                {
                    Toast toast1 = Toast.makeText(getApplicationContext(), "Existen campos vacios. Revise el formulario", Toast.LENGTH_SHORT);
                    toast1.setGravity(Gravity.CENTER,0,0);
                    toast1.show();
                }
                break;
            case R.id.b_cancelar:
                //Se cancela el formulario se resetea.

                break;
            default:
                break;

        }
    }

    public void limpiarCampos()
    {

    }
}