package edu.uoc.android.gfp;

import android.app.Application;
import android.content.Context;
import android.support.annotation.NonNull;
import android.widget.ArrayAdapter;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.GenericTypeIndicator;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;
import com.orm.SugarContext;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


import edu.uoc.android.gfp.modeloDatos.T_Contribucion;
import edu.uoc.android.gfp.modeloDatos.T_Cuenta_Bancaria;
import edu.uoc.android.gfp.modeloDatos.T_Deseo;
import edu.uoc.android.gfp.modeloDatos.T_Gasto;
import edu.uoc.android.gfp.modeloDatos.T_Ingreso;
import edu.uoc.android.gfp.modeloDatos.T_Periodicidad;
import edu.uoc.android.gfp.modeloDatos.T_Presupuesto;
import edu.uoc.android.gfp.modeloDatos.T_Situacion;
import edu.uoc.android.gfp.modeloDatos.T_Usuario;
import edu.uoc.android.gfp.modeloDatos.T_tipo_Alimentacion;
import edu.uoc.android.gfp.modeloDatos.T_tipo_Gasto;

import edu.uoc.android.gfp.modeloDatos.T_tipo_Ingreso;
import edu.uoc.android.gfp.modeloDatos.T_tipo_Necesidad;
import edu.uoc.android.gfp.modeloDatos.T_tipo_Periodicidad;

public class MyApplication extends Application {

    //Propiedad para conectarse a Firebase de google
    public FirebaseAuth mAuth;
    public FirebaseDatabase database;

    //Variable que utilizamos para indicar si hay conexión a Internet
    public boolean conexOnline;

    //Propiedad para BD
    public DatabaseReference objBD;

    public void onCreate() {
        super.onCreate();
        SugarContext.init(this.getApplicationContext());

        /*-------------------------------------------------------------------
        ---------------------------------------------------------------------
        -------------INFO RELATIVA A LA CONEXIÓN CON LA BD ------------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
         */

        conexOnline = false;

        mAuth = FirebaseAuth.getInstance();
        database = FirebaseDatabase.getInstance();

        //Autenticación contra la BD de firebase
        mAuth.signInWithEmailAndPassword("fgg2@alu.ua.es", "12345678")
                .addOnCompleteListener(new OnCompleteListener<AuthResult>() {
                    @Override
                    public void onComplete(@NonNull Task<AuthResult> task) {

                        if (!task.isSuccessful()) {
                            //Si no hay conexión con el servidor enviamos una notificación a la app
                            //Indicando que la señal ha fallado
                            conexOnline = false;
                        } else
                            conexOnline = true;
                    }
                });


        inicializacionBD();

        //Actualiza la información relativa a la BD
        actualizaInfoBDLocal();


    }



    public void inicializacionBD()
    {
        //si la app es la primera vez que se instala
        //debe tener registros en las tipologias

        if (T_tipo_Gasto.obtenerRecuentoTiposGastos()==0)
        {
                T_tipo_Gasto.altaTipoGasto(getApplicationContext(),"Gasto Genérico",1,"Primaria");
        }

        if (T_tipo_Ingreso.obtenerRecuentoTiposIngresos()==0)
        {
                T_tipo_Ingreso.altaTipoIngreso(getApplicationContext(),"Ingreso Genérico");
        }

        if (T_tipo_Necesidad.obtenerRecuentoTiposNecesidad()==0)
        {
                T_tipo_Necesidad.altaTipoNecesidad(getApplicationContext(),"Primaria");
                T_tipo_Necesidad.altaTipoNecesidad(getApplicationContext(),"Secundaria");
                T_tipo_Necesidad.altaTipoNecesidad(getApplicationContext(),"Terciaria");

        }

        if (T_tipo_Periodicidad.obtenerRecuentoTiposPeriodicidad()==0)
        {
            T_tipo_Periodicidad.altaTipoPeriodicidad(getApplicationContext(),"Mensual");
            T_tipo_Periodicidad.altaTipoPeriodicidad(getApplicationContext(),"Trimestral");
            T_tipo_Periodicidad.altaTipoPeriodicidad(getApplicationContext(),"Anual");
            T_tipo_Periodicidad.altaTipoPeriodicidad(getApplicationContext(),"Esporádica");

        }

        if (T_tipo_Alimentacion.obtenerRecuentoTiposAlimentacion()==0)
        {
            T_tipo_Alimentacion.altaTipoAlimentacion(getApplicationContext(),"Manual");
            T_tipo_Alimentacion.altaTipoAlimentacion(getApplicationContext(),"Automática");
            T_tipo_Alimentacion.altaTipoAlimentacion(getApplicationContext(),"Manual/Automática");

        }
    }





        /*-------------------------------------------------------------------
        ---------------------------------------------------------------------
        -------------SINCRONIZA LA INFORMACIÓN DE FB CON LA BD LOCAL --------
        -------------SE TOMA COMO REFERENCIA LA BD LOCAL---------------------
        -------------SE ACTUALIZA FIREBASE-----------------------------------
        ---------------------------------------------------------------------
         */

    public void actualizaInfoBDLocal() {

        int indice;
        String item;

        // GASTOS
        objBD = database.getReference("Gasto");
        //Se eliminan los gastos que puedan haber registrados
        objBD.removeValue();
        List<T_Gasto.Gasto> GASTOS = T_Gasto.obtenerTodosGasto();
        for ( indice = 0; indice < GASTOS.size(); indice++) {
            item=objBD.child(GASTOS.get(indice).getIdGasto().toString()).getKey();
            objBD.child(item).setValue(GASTOS.get(indice));
        }

        // INGRESOS
        objBD = database.getReference("Ingreso");
        //Se eliminan los ingresos que puedan haber registrados
        objBD.removeValue();
        List<T_Ingreso.Ingreso> INGRESOS = T_Ingreso.obtenerTodosIngresos();
        for ( indice = 0; indice < INGRESOS.size(); indice++) {
            item=objBD.child(INGRESOS.get(indice).getIdIngreso().toString()).getKey();
            objBD.child(item).setValue(INGRESOS.get(indice));
        }

        // PRESUPUESTOS
        objBD = database.getReference("Presupuesto");
        //Se eliminan los presupuestos que puedan haber registrados
        objBD.removeValue();
        List<T_Presupuesto.Presupuesto> PRESUPUESTOS = T_Presupuesto.obtenerTodosPresupuestos();
        for ( indice = 0; indice < PRESUPUESTOS.size(); indice++) {
            item=objBD.child(PRESUPUESTOS.get(indice).getIdPresupuesto().toString()).getKey();
            objBD.child(item).setValue(PRESUPUESTOS.get(indice));
        }


        // DESEOS
        objBD = database.getReference("Deseo");
        //Se eliminan los deseos que puedan haber registrados
        objBD.removeValue();
        List<T_Deseo.Deseo> DESEOS = T_Deseo.obtenerTodosDeseos();
        for ( indice = 0; indice < DESEOS.size(); indice++) {
            item=objBD.child(DESEOS.get(indice).getIdDeseo().toString()).getKey();
            objBD.child(item).setValue(DESEOS.get(indice));
        }

        // SITUACION
        objBD = database.getReference("Situacion");
        //Se eliminan las Situaciones que puedan haber registrados
        objBD.removeValue();
        List<T_Situacion.Situacion> SITUACIONES = T_Situacion.obtenerTodasSituaciones();
        for ( indice = 0; indice < SITUACIONES.size(); indice++) {
            item=objBD.child(SITUACIONES.get(indice).getIdSituacion().toString()).getKey();
            objBD.child(item).setValue(SITUACIONES.get(indice));
        }

        // USUARIO
        objBD = database.getReference("Usuario");
        //Se eliminan los Usuarios que puedan haber registrados
        objBD.removeValue();
        List<T_Usuario.Usuario> USUARIOS = T_Usuario.obtenerTodosUsuarios();
        for ( indice = 0; indice < USUARIOS.size(); indice++) {
            item=objBD.child(USUARIOS.get(indice).getidUsuario().toString()).getKey();
            objBD.child(item).setValue(USUARIOS.get(indice));
        }

        // TIPO_GASTO
        objBD = database.getReference("tipo_gasto");
        //Se eliminan los tipos de gasto que puedan haber registrados
        objBD.removeValue();
        List<T_tipo_Gasto.Tipo_Gasto> TIPOS_GASTOS = T_tipo_Gasto.obtenerTodosTiposGasto();
        for ( indice = 0; indice < TIPOS_GASTOS.size(); indice++) {
            item=objBD.child(TIPOS_GASTOS.get(indice).getidTipoGasto().toString()).getKey();
            objBD.child(item).setValue(TIPOS_GASTOS.get(indice));
        }

        // TIPO_INGRESO
        objBD = database.getReference("tipo_ingreso");
        //Se eliminan los tipos de ingreso que puedan haber registrados
        objBD.removeValue();
        List<T_tipo_Ingreso.Tipo_Ingreso> TIPOS_INGRESOS = T_tipo_Ingreso.obtenerTodosTiposIngresos();
        for ( indice = 0; indice < TIPOS_INGRESOS.size(); indice++) {
            item=objBD.child(TIPOS_INGRESOS.get(indice).getidTipoIngreso().toString()).getKey();
            objBD.child(item).setValue(TIPOS_INGRESOS.get(indice));
        }

        // TIPO_PERIODICIDAD
        objBD = database.getReference("tipo_periodicidad");
        //Se eliminan los tipos de periodicidad que puedan haber registrados
        objBD.removeValue();
        List<T_Periodicidad.Tipo_Periodicidad> TIPOS_PERIODICIDAD = T_Periodicidad.obtenerTodosPeriodicidad();
        for ( indice = 0; indice < TIPOS_PERIODICIDAD.size(); indice++) {
            item=objBD.child(TIPOS_PERIODICIDAD.get(indice).getIdPeriodicidad().toString()).getKey();
            objBD.child(item).setValue(TIPOS_PERIODICIDAD.get(indice));
        }

        // NECESIDAD
        objBD = database.getReference("tipo_necesidad");
        //Se eliminan los tipos de necesidad que puedan haber registrados
        objBD.removeValue();
        List<T_tipo_Necesidad.Tipo_Necesidad> TIPOS_NECESIDAD = T_tipo_Necesidad.obtenerTodosTiposNecesidad();
        for ( indice = 0; indice < TIPOS_NECESIDAD.size(); indice++) {
            item=objBD.child(TIPOS_NECESIDAD.get(indice).getidNecesidad().toString()).getKey();
            objBD.child(item).setValue(TIPOS_NECESIDAD.get(indice));
        }

        // CUENTA BANCARIA
        objBD = database.getReference("Cuenta_Bancaria");
        //Se eliminan las cuentas bancarias que puedan haber registrados
        objBD.removeValue();
        List<T_Cuenta_Bancaria.Cuenta_Bancaria> CUENTAS_BANCARIAS = T_Cuenta_Bancaria.obtenerTodosCuentasBancarias();
        for ( indice = 0; indice < CUENTAS_BANCARIAS.size(); indice++) {
            item=objBD.child(CUENTAS_BANCARIAS.get(indice).getIdCuenta().toString()).getKey();
            objBD.child(item).setValue(CUENTAS_BANCARIAS.get(indice));
        }

        // CONTRIBUCION
        objBD = database.getReference("Contribucion");
        //Se eliminan las contribuciones que puedan haber registrados
        objBD.removeValue();
        List<T_Contribucion.Contribucion> CONTRIBUCIONES = T_Contribucion.obtenerTodosContribuciones();
        for ( indice = 0; indice < CONTRIBUCIONES.size(); indice++) {
            item=objBD.child(CONTRIBUCIONES.get(indice).getIdContribucion().toString()).getKey();
            objBD.child(item).setValue(CONTRIBUCIONES.get(indice));
        }

        // TIPO_ALIMENTACION
        objBD = database.getReference("tipo_alimentacion");
        //Se eliminan los tipos de necesidad que puedan haber registrados
        objBD.removeValue();
        List<T_tipo_Alimentacion.Tipo_Alimentacion> TIPOS_ALIMENTACION = T_tipo_Alimentacion.obtenerTodosTiposAlimentacion();
        for ( indice = 0; indice < TIPOS_ALIMENTACION.size(); indice++) {
            item=objBD.child(TIPOS_ALIMENTACION.get(indice).getidAlimentacion().toString()).getKey();
            objBD.child(item).setValue(TIPOS_ALIMENTACION.get(indice));
        }

    }





        /*-------------------------------------------------------------------
        ---------------------------------------------------------------------
        -------------INFO RELATIVA AL OBJETO TIPO_GASTO... ------------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
         */


    //obtener Tipos de Gasto de la BD
    private ArrayList<String> obtenerTipoGastoBD() {

        List<T_tipo_Gasto.Tipo_Gasto> values = T_tipo_Gasto.obtenerTodosTiposGasto();
        ArrayList<String> tiposGastos = new ArrayList<String>();

        Iterator<T_tipo_Gasto.Tipo_Gasto> it = values.iterator();
        T_tipo_Gasto.Tipo_Gasto tipo_gasto_it;
        // mientras queden tipos de gasto
        while (it.hasNext()) {
            tipo_gasto_it = it.next();
            if (tipo_gasto_it != null) {
                tiposGastos.add(tipo_gasto_it.getdescripcion());
            }
        }

        return (tiposGastos);
    }

    //Carga desplegable de Necesidad
    public ArrayAdapter<String> cargaNecesidad() {
        ArrayAdapter<String> dataAdapter = null;

        //Si no hay conexión se obtiene de BD local
        ArrayList<String> necesidades = new ArrayList<String>();
        necesidades.add(getResources().getString(R.string.ini_spinner));

        List<T_tipo_Necesidad.Tipo_Necesidad> tn_bd = T_tipo_Necesidad.obtenerTodosTiposNecesidad();

        for (int indice = 0; indice < tn_bd.size(); indice++) {
            necesidades.add(tn_bd.get(indice).descripcion);
        }

        dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, necesidades);

        return dataAdapter;
    }

    //Carga desplegable de TipoGasto
    public ArrayAdapter<String> cargaTipoGasto() {
        ArrayAdapter<String> dataAdapter = null;

        //Si no hay conexión se obtiene de BD local
        ArrayList<String> tiposGastos = new ArrayList<String>();
        tiposGastos.add(getResources().getString(R.string.ini_spinner));

        List<T_tipo_Gasto.Tipo_Gasto> tp_bd = T_tipo_Gasto.obtenerTodosTiposGasto();

        for (int indice = 0; indice < tp_bd.size(); indice++) {
            tiposGastos.add(tp_bd.get(indice).descripcion);
        }

        dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, tiposGastos);

        return dataAdapter;
    }


    //Carga desplegable de TipoGasto
    public ArrayAdapter<String> cargaTipoGastoFB() {

        final ArrayList<String> tiposGastos = new ArrayList<String>();
        tiposGastos.add(getResources().getString(R.string.ini_spinner));

        objBD = database.getReference("Tipo_Gasto");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                GenericTypeIndicator<ArrayList<T_tipo_Gasto.Tipo_Gasto>> genericTypeIndicator =
                        new GenericTypeIndicator<ArrayList<T_tipo_Gasto.Tipo_Gasto>>() {
                        };

                ArrayList<T_tipo_Gasto.Tipo_Gasto> values = dataSnapshot.getValue(genericTypeIndicator);

                Iterator<T_tipo_Gasto.Tipo_Gasto> it = values.iterator();
                T_tipo_Gasto.Tipo_Gasto gasto_it;
                // mientras queden tipos de gasto
                while (it.hasNext()) {
                    gasto_it = it.next();

                    if (gasto_it != null) {
                        tiposGastos.add(gasto_it.getdescripcion());

                    }
                }


            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


        ArrayAdapter<String> dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, tiposGastos);

        return dataAdapter;
    }


    public Integer altaTipoGasto(String descripcion,Integer idNecesidad, String necesidad) {
        Integer idTipoGasto;

        idTipoGasto = T_tipo_Gasto.altaTipoGasto(getApplicationContext(), descripcion,idNecesidad,necesidad);
        if (conexOnline)
            altaTipoGastoFB(idTipoGasto,descripcion,idNecesidad,necesidad);

        return idTipoGasto;
    }


    public void altaTipoGastoFB(Integer idTipoGasto,String descripcion,Integer idNecesidad, String necesidad) {

        objBD = database.getReference("Tipo_Gasto");

        T_tipo_Gasto.Tipo_Gasto nTipoGasto = new T_tipo_Gasto.Tipo_Gasto (idTipoGasto, descripcion, idNecesidad,necesidad);
        String item = objBD.child(idTipoGasto.toString()).getKey();
        objBD.child(item).setValue(nTipoGasto);
    }

    public void modificaTipoGasto(Integer idTipoGasto, String descripcion,Integer idNecesidad,String necesidad) {

        T_tipo_Gasto.modificaTipoGasto(getApplicationContext(), idTipoGasto,descripcion,idNecesidad,necesidad);

        if (conexOnline)
            modificaTipoGastoFB(idTipoGasto, descripcion,idNecesidad, necesidad);

    }


    public void modificaTipoGastoFB(Integer idTipoGasto, String descripcion,Integer idNecesidad, String necesidad) {

        objBD = database.getReference("Tipo_Gasto");

        Map<String, T_tipo_Gasto.Tipo_Gasto> nTipoGasto = new HashMap<>();
        nTipoGasto.put(idTipoGasto.toString(), new T_tipo_Gasto.Tipo_Gasto(idTipoGasto, descripcion, idNecesidad, necesidad));

        objBD.setValue(nTipoGasto);
    }


    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO TIPO PERIODICIDAD --------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */


    //Carga desplegable de TipoPeriodicidad

    public ArrayAdapter<String> cargaTipoPeriodicidad() {
        ArrayAdapter<String> dataAdapter = null;

        ArrayList<String> tiposPeriodicidad = new ArrayList<String>();
        tiposPeriodicidad.add(getResources().getString(R.string.ini_spinner));

        List<T_tipo_Periodicidad.Tipo_Periodicidad> tp_bd = T_tipo_Periodicidad.obtenerTodosTiposPeriodicidad();

        for (int indice = 0; indice < tp_bd.size(); indice++) {
            tiposPeriodicidad.add(tp_bd.get(indice).descripcion);
        }

        dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, tiposPeriodicidad);


        return dataAdapter;
    }


    //Carga desplegable de TipoGasto
    public ArrayAdapter<String> cargaTipoPeriodicidadFB() {

        final ArrayList<String> tiposPeriodicidad = new ArrayList<String>();
        tiposPeriodicidad.add(getResources().getString(R.string.ini_spinner));

        objBD = database.getReference("Tipo_Periodicidad");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                GenericTypeIndicator<ArrayList<T_Periodicidad.Tipo_Periodicidad>> genericTypeIndicator =
                        new GenericTypeIndicator<ArrayList<T_Periodicidad.Tipo_Periodicidad>>() {
                        };

                ArrayList<T_Periodicidad.Tipo_Periodicidad> values = dataSnapshot.getValue(genericTypeIndicator);

                Iterator<T_Periodicidad.Tipo_Periodicidad> it = values.iterator();
                T_Periodicidad.Tipo_Periodicidad periodicidad_it;
                // mientras queden tipos de gasto
                while (it.hasNext()) {
                    periodicidad_it = it.next();

                    if (periodicidad_it != null) {
                        tiposPeriodicidad.add(periodicidad_it.getdescripcion());
                    }
                }


            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });

        ArrayAdapter<String> dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, tiposPeriodicidad);

        return dataAdapter;
    }



    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO GASTO --------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */


    public Integer altaGasto(String descripcion, Integer idTipoGasto, float importe, String fechaGasto, Integer idPeriodicidad, Integer idNecesidad, String idUsuario,
                             String tipoGasto, String periodicidad, String necesidad) {
        Integer idGasto;

        idGasto = T_Gasto.altaGasto(getApplicationContext(), descripcion, idTipoGasto, importe, fechaGasto, idPeriodicidad, idNecesidad, idUsuario,
                tipoGasto, periodicidad, necesidad);
        if (conexOnline)
            altaGastoFB(idGasto, descripcion, idTipoGasto, importe, fechaGasto, idPeriodicidad, idNecesidad, idUsuario, tipoGasto, periodicidad, necesidad);

        return idGasto;
    }


    public void altaGastoFB(Integer idGasto, String descripcion, Integer idTipoGasto, float importe, String fechaGasto, Integer idPeriodicidad, Integer idNecesidad, String idUsuario,
                            String tipoGasto, String periodicidad, String necesidad) {

        objBD = database.getReference("Gasto");

        //objBD.child(idGasto.toString()).push().setValue(new T_Gasto.Gasto(idGasto,  descripcion,  idTipoGasto,  importe,  fechaGasto,  idPeriodicidad,  idNecesidad,  idUsuario,tipoGasto,periodicidad,necesidad));

        //Map<String, T_Gasto.Gasto> nGasto = new HashMap<>();
        //nGasto.put(idGasto.toString(),new T_Gasto.Gasto(idGasto,  descripcion,  idTipoGasto,  importe,  fechaGasto,  idPeriodicidad,  idNecesidad,  idUsuario,tipoGasto,periodicidad,necesidad));
        //objBD.setValue(nGasto);


        //objBD.push(idGasto.toString(),new T_Gasto.Gasto(idGasto,  descripcion,  idTipoGasto,  importe,  fechaGasto,  idPeriodicidad,  idNecesidad,  idUsuario,tipoGasto,periodicidad,necesidad));

        //objBD.child(idGasto.toString()).push().setValue(new T_Gasto.Gasto(idGasto,  descripcion,  idTipoGasto,  importe,  fechaGasto,  idPeriodicidad,  idNecesidad,  idUsuario,tipoGasto,periodicidad,necesidad));

        T_Gasto.Gasto nGasto = new T_Gasto.Gasto(idGasto, descripcion, idTipoGasto, importe, fechaGasto, idPeriodicidad, idNecesidad, idUsuario, tipoGasto, periodicidad, necesidad);
        String item = objBD.child(idGasto.toString()).getKey();
        objBD.child(item).setValue(nGasto);
    }

    public void modificaGasto(Integer idGasto, String descripcion, Integer idTipoGasto, float importe, String fechaGasto, Integer idPeriodicidad, Integer idNecesidad, String idUsuario,
                              String tipoGasto, String periodicidad, String necesidad) {

        T_Gasto.modificaGasto(getApplicationContext(), idGasto, descripcion, idTipoGasto, importe, fechaGasto, idPeriodicidad, idNecesidad, idUsuario,
                tipoGasto, periodicidad, necesidad);

        if (conexOnline)
            modificaGastoFB(idGasto, descripcion, idTipoGasto, importe, fechaGasto, idPeriodicidad, idNecesidad, idUsuario, tipoGasto, periodicidad, necesidad);

    }


    public void modificaGastoFB(Integer idGasto, String descripcion, Integer idTipoGasto, float importe, String fechaGasto, Integer idPeriodicidad, Integer idNecesidad, String idUsuario,
                                String tipoGasto, String periodicidad, String necesidad) {
        objBD = database.getReference("Gasto");

        Map<String, T_Gasto.Gasto> nGasto = new HashMap<>();
        nGasto.put(idGasto.toString(), new T_Gasto.Gasto(idGasto, descripcion, idTipoGasto, importe, fechaGasto, idPeriodicidad, idNecesidad, idUsuario, tipoGasto, periodicidad, necesidad));

        objBD.setValue(nGasto);
    }


    public void bajaGasto(Integer idGasto) {

        //Se elimina de la base de datos Local y tambien se elimina de FB
        T_Gasto.eliminarGasto(getApplicationContext(), idGasto);

        //Se elimina el gasto de FB también
        bajaGastoFB(idGasto);

    }


    public void bajaGastoFB(Integer idGasto) {
        //Eliminar de la BD de FB el gasto indicado
        objBD = database.getReference("Gasto");


        objBD.child(idGasto.toString()).removeValue();

    }



        /*-------------------------------------------------------------------
        ---------------------------------------------------------------------
        -------------INFO RELATIVA AL OBJETO TIPO_INGRESO...-----------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
         */


    private ArrayList<String> obtenerTipoIngresoBD() {

        List<T_tipo_Ingreso.Tipo_Ingreso> values = T_tipo_Ingreso.obtenerTodosTiposIngresos();
        ArrayList<String> tiposIngresos = new ArrayList<String>();

        Iterator<T_tipo_Ingreso.Tipo_Ingreso> it = values.iterator();
        T_tipo_Ingreso.Tipo_Ingreso tipo_ingreso_it;
        // mientras queden tipos de ingresos
        while (it.hasNext()) {
            tipo_ingreso_it = it.next();
            if (tipo_ingreso_it != null) {
                tiposIngresos.add(tipo_ingreso_it.getdescripcion());
            }
        }

        return (tiposIngresos);
    }

    //Carga desplegable de TipoIngreso
    public ArrayAdapter<String> cargaTipoIngreso() {
        ArrayAdapter<String> dataAdapter = null;

        //Si no hay conexión se obtiene de BD local
        ArrayList<String> tiposIngresos = new ArrayList<String>();
        tiposIngresos.add(getResources().getString(R.string.ini_spinner));

        List<T_tipo_Ingreso.Tipo_Ingreso> ti_bd = T_tipo_Ingreso.obtenerTodosTiposIngresos();

        for (int indice = 0; indice < ti_bd.size(); indice++) {
            tiposIngresos.add(ti_bd.get(indice).descripcion);
        }

        dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, tiposIngresos);

        return dataAdapter;
    }


    //Carga desplegable de TipoGasto
    public ArrayAdapter<String> cargaTipoIngresoFB() {

        final ArrayList<String> tiposIngresos = new ArrayList<String>();
        tiposIngresos.add(getResources().getString(R.string.ini_spinner));

        objBD = database.getReference("Tipo_Ingreso");
        // Lee de la base de datos si se produce alguna acutalizacion

        objBD.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                GenericTypeIndicator<ArrayList<T_tipo_Ingreso.Tipo_Ingreso>> genericTypeIndicator =
                        new GenericTypeIndicator<ArrayList<T_tipo_Ingreso.Tipo_Ingreso>>() {
                        };

                ArrayList<T_tipo_Ingreso.Tipo_Ingreso> values = dataSnapshot.getValue(genericTypeIndicator);

                Iterator<T_tipo_Ingreso.Tipo_Ingreso> it = values.iterator();
                T_tipo_Ingreso.Tipo_Ingreso ingreso_it;


                while (it.hasNext()) {
                    ingreso_it = it.next();

                    if (ingreso_it != null) {
                        tiposIngresos.add(ingreso_it.getdescripcion());

                    }
                }


            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


        ArrayAdapter<String> dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, tiposIngresos);

        return dataAdapter;
    }


    public Integer altaTipoIngreso(String descripcion) {
        Integer idTipoIngreso;

        idTipoIngreso = T_tipo_Ingreso.altaTipoIngreso(getApplicationContext(), descripcion);
        if (conexOnline)
            altaTipoIngresoFB(idTipoIngreso,descripcion);

        return idTipoIngreso;
    }


    public void altaTipoIngresoFB(Integer idTipoIngreso,String descripcion) {

        objBD = database.getReference("Tipo_Ingreso");

        T_tipo_Ingreso.Tipo_Ingreso nTipoIngreso = new T_tipo_Ingreso.Tipo_Ingreso (idTipoIngreso, descripcion);
        String item = objBD.child(idTipoIngreso.toString()).getKey();
        objBD.child(item).setValue(nTipoIngreso);
    }

    public void modificaTipoIngreso(Integer idTipoIngreso, String descripcion) {

        T_tipo_Ingreso.modificaTipoIngreso(getApplicationContext(), idTipoIngreso,descripcion);

        if (conexOnline)
          modificaTipoIngresoFB(idTipoIngreso, descripcion);

    }


    public void modificaTipoIngresoFB(Integer idTipoIngreso, String descripcion) {

        objBD = database.getReference("Tipo_Ingreso");

        Map<String, T_tipo_Ingreso.Tipo_Ingreso> nTipoIngreso = new HashMap<>();
        nTipoIngreso.put(idTipoIngreso.toString(), new T_tipo_Ingreso.Tipo_Ingreso(idTipoIngreso, descripcion));

        objBD.setValue(nTipoIngreso);
    }


    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO INGRESO --------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */


    public void bajaIngreso(Integer idIngreso) {

        //Se elimina de la base de datos Local y tambien se elimina de FB
        T_Ingreso.eliminarIngreso(getApplicationContext(), idIngreso);

        if (conexOnline)
            //Se elimina el gasto de FB también
            bajaIngresoFB(idIngreso);

    }


    public void bajaIngresoFB(Integer idIngreso) {
        //Eliminar de la BD de FB el deseo indicado
        objBD = database.getReference("Ingreso");


        objBD.child(idIngreso.toString()).removeValue();


    }

    public Integer altaIngreso(String descripcion, Integer idTipoIngreso, float importe, String fechaIngreso, Integer idPeriodicidad, String idUsuario,
                               String tipoIngreso, String periodicidad) {
        Integer idIngreso;

        idIngreso = T_Ingreso.altaIngreso(getApplicationContext(), descripcion, idTipoIngreso, importe, fechaIngreso, idPeriodicidad, idUsuario,
                tipoIngreso, periodicidad);

        if (conexOnline)
            altaIngresoFB(idIngreso, descripcion, idTipoIngreso, importe, fechaIngreso, idPeriodicidad, idUsuario, tipoIngreso, periodicidad);

        return idIngreso;
    }


    public void altaIngresoFB(Integer idIngreso, String descripcion, Integer idTipoIngreso, float importe, String fechaIngreso, Integer idPeriodicidad, String idUsuario,
                              String tipoIngreso, String periodicidad) {
        objBD = database.getReference("Ingreso");

        T_Ingreso.Ingreso nIngreso = new T_Ingreso.Ingreso(idIngreso, descripcion, idTipoIngreso, importe, fechaIngreso, idPeriodicidad, idUsuario, tipoIngreso, periodicidad);
        String item = objBD.child(idIngreso.toString()).getKey();
        objBD.child(item).setValue(nIngreso);

    }

    public void modificaIngreso(Integer idIngreso, String descripcion, Integer idTipoIngreso, float importe, String fechaIngreso, Integer idPeriodicidad, String idUsuario,
                                String tipoIngreso, String periodicidad) {

        T_Ingreso.modificaIngreso(getApplicationContext(), idIngreso, descripcion, idTipoIngreso, importe, fechaIngreso, idPeriodicidad, idUsuario,
                tipoIngreso, periodicidad);

        if (conexOnline)
            modificaIngresoFB(idIngreso, descripcion, idTipoIngreso, importe, fechaIngreso, idPeriodicidad, idUsuario, tipoIngreso, periodicidad);

    }


    public void modificaIngresoFB(Integer idIngreso, String descripcion, Integer idTipoIngreso, float importe, String fechaIngreso, Integer idPeriodicidad, String idUsuario,
                                  String tipoIngreso, String periodicidad) {
        objBD = database.getReference("Ingreso");

        Map<String, T_Ingreso.Ingreso> nIngreso = new HashMap<>();
        nIngreso.put(idIngreso.toString(), new T_Ingreso.Ingreso(idIngreso, descripcion, idTipoIngreso, importe, fechaIngreso, idPeriodicidad, idUsuario, tipoIngreso, periodicidad));

        objBD.setValue(nIngreso);
    }



    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO CONTRIBUCION--------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */


    public void bajaContribucion(Integer idContribucion) {

        //Se elimina de la base de datos Local y tambien se elimina de FB
        T_Contribucion.eliminarContribucion(getApplicationContext(), idContribucion);

        if (conexOnline)
            //Se elimina el gasto de FB también
            bajaContribucionFB(idContribucion);

    }


    public void bajaContribucionFB(Integer idContribucion) {
        //Eliminar de la BD de FB contribucion
        objBD = database.getReference("Contribucion");


        objBD.child(idContribucion.toString()).removeValue();


    }

    public Integer altaContribucion(String descripcion, Integer idDeseo, float importe, String fechaContribucion, String idUsuario,
                                    String deseo) {
        Integer idContribucion;

        idContribucion = T_Contribucion.altaContribucion(getApplicationContext(), descripcion, idDeseo, importe, fechaContribucion, idUsuario,
                deseo);

        if (conexOnline)
            altaContribucionFB(idContribucion, descripcion, idDeseo, importe, fechaContribucion, idUsuario, deseo);

        return idContribucion;
    }


    public void altaContribucionFB(Integer idContribucion, String descripcion, Integer idDeseo, float importe, String fechaContribucion, String idUsuario,
                                   String deseo) {
        objBD = database.getReference("Contribucion");

        T_Contribucion.Contribucion nContribucion = new T_Contribucion.Contribucion(idContribucion, descripcion, importe, idDeseo, fechaContribucion, idUsuario, deseo);
        String item = objBD.child(idContribucion.toString()).getKey();
        objBD.child(item).setValue(nContribucion);

    }

    public void modificaContribucion(Integer idContribucion, String descripcion, Integer idDeseo, float importe, String fechaContribucion, String idUsuario,
                                     String deseo) {

        T_Contribucion.modificaContribucion(getApplicationContext(), idContribucion, descripcion, idDeseo, importe, fechaContribucion, idUsuario,
                deseo);

        if (conexOnline)
            modificaContribucionFB(idContribucion, descripcion, idDeseo, importe, fechaContribucion, idUsuario, deseo);

    }


    public void modificaContribucionFB(Integer idContribucion, String descripcion, Integer idDeseo, float importe, String fechaContribucion, String idUsuario,
                                       String deseo) {
        objBD = database.getReference("Contribucion");

        Map<String, T_Contribucion.Contribucion> nContribucion = new HashMap<>();
        nContribucion.put(idContribucion.toString(), new T_Contribucion.Contribucion(idContribucion, descripcion, importe, idDeseo, fechaContribucion, idUsuario, deseo));

        objBD.setValue(nContribucion);
    }




    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO DESEO---------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */


    //Carga desplegable de Deseo
    public ArrayAdapter<String> cargaDeseo(String idUsuario) {
        ArrayAdapter<String> dataAdapter = null;

        //Si no hay conexión se obtiene de BD local
        ArrayList<String> deseos = new ArrayList<String>();
        deseos.add(getResources().getString(R.string.ini_spinner));

        List<T_Deseo.Deseo> deseos_bd = T_Deseo.obtenerTodosDeseos(idUsuario);

        for (int indice = 0; indice < deseos_bd.size(); indice++) {
            deseos.add(deseos_bd.get(indice).descripcion);
        }

        dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, deseos);

        return dataAdapter;
    }


    public Integer altaDeseo(String descripcion, float importe, String desdeFecha, String hastaFecha, String idUsuario) {
        Integer idDeseo;

        idDeseo = T_Deseo.altaDeseo(getApplicationContext(), descripcion, importe, desdeFecha, hastaFecha, idUsuario);

        if (conexOnline)
            altaDeseoFB(idDeseo, descripcion, importe, desdeFecha, hastaFecha, idUsuario);

        return idDeseo;
    }


    public void altaDeseoFB(Integer idDeseo, String descripcion, float importe, String desdeFecha, String hastaFecha, String idUsuario) {

        objBD = database.getReference("Deseo");

        T_Deseo.Deseo nDeseo = new T_Deseo.Deseo(idDeseo, descripcion, importe, desdeFecha, hastaFecha, idUsuario);
        String item = objBD.child(idDeseo.toString()).getKey();
        objBD.child(item).setValue(nDeseo);
    }

    public void modificaDeseo(Integer idDeseo, String descripcion, float importe, String desdeFecha, String hastaFecha, String idUsuario) {

        T_Deseo.modificaDeseo(getApplicationContext(), idDeseo, descripcion, importe, desdeFecha, hastaFecha, idUsuario);

        if (conexOnline)
            modificaDeseoFB(idDeseo, descripcion, importe, desdeFecha, hastaFecha, idUsuario);

    }


    public void modificaDeseoFB(Integer idDeseo, String descripcion, float importe, String desdeFecha, String hastaFecha, String idUsuario) {
        objBD = database.getReference("Deseo");

        Map<String, T_Deseo.Deseo> nDeseo = new HashMap<>();
        nDeseo.put(idDeseo.toString(), new T_Deseo.Deseo(idDeseo, descripcion, importe, desdeFecha, hastaFecha, idUsuario));

        objBD.setValue(nDeseo);
    }


    public void bajaDeseo(Integer idDeseo) {

        //Se elimina de la base de datos Local y tambien se elimina de FB
        T_Deseo.eliminarDeseo(getApplicationContext(), idDeseo);

        //Se elimina el deseo de FB también
        bajaDeseoFB(idDeseo);

    }


    public void bajaDeseoFB(Integer idDeseo) {
        //Eliminar de la BD de FB el deseo indicado
        objBD = database.getReference("Deseo");


        objBD.child(idDeseo.toString()).removeValue();

    }



    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO PRESUPUESTO---------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */


    public Integer altaPresupuesto(String descripcion, float importe, String desdeFecha, String hastaFecha, String idUsuario) {
        Integer idPresupuesto;

        idPresupuesto = T_Presupuesto.altaPresupuesto(getApplicationContext(), descripcion, importe, desdeFecha, hastaFecha, idUsuario);

        if (conexOnline)
            altaPresupuestoFB(idPresupuesto, descripcion, importe, desdeFecha, hastaFecha, idUsuario);

        return idPresupuesto;
    }


    public void altaPresupuestoFB(Integer idPresupuesto, String descripcion, float importe, String desdeFecha, String hastaFecha, String idUsuario) {

        objBD = database.getReference("Presupuesto");

        T_Presupuesto.Presupuesto nPresupuesto = new T_Presupuesto.Presupuesto(idPresupuesto, descripcion, importe, desdeFecha, hastaFecha, idUsuario);
        String item = objBD.child(idPresupuesto.toString()).getKey();
        objBD.child(item).setValue(nPresupuesto);
    }

    public void modificaPresupuesto(Integer idPresupuesto, String descripcion, float importe, String desdeFecha, String hastaFecha, String idUsuario) {

        T_Presupuesto.modificaPresupuesto(getApplicationContext(), idPresupuesto, descripcion, importe, desdeFecha, hastaFecha, idUsuario);

        if (conexOnline)
            modificaPresupuestoFB(idPresupuesto, descripcion, importe, desdeFecha, hastaFecha, idUsuario);

    }


    public void modificaPresupuestoFB(Integer idPresupuesto, String descripcion, float importe, String desdeFecha, String hastaFecha, String idUsuario) {
        objBD = database.getReference("Presupuesto");

        Map<String, T_Presupuesto.Presupuesto> nPresupuesto = new HashMap<>();
        nPresupuesto.put(idPresupuesto.toString(), new T_Presupuesto.Presupuesto(idPresupuesto, descripcion, importe, desdeFecha, hastaFecha, idUsuario));

        objBD.setValue(nPresupuesto);
    }


    public void bajaPresupuesto(Integer idPresupuesto) {

        //Se elimina de la base de datos Local y tambien se elimina de FB
        T_Presupuesto.eliminarPresupuesto(getApplicationContext(), idPresupuesto);

        //Se elimina el presupuesto de FB también
        bajaPresupuestoFB(idPresupuesto);

    }


    public void bajaPresupuestoFB(Integer idPresupuesto) {
        //Eliminar de la BD de FB el presupuesto indicado
        objBD = database.getReference("Presupuesto");


        objBD.child(idPresupuesto.toString()).removeValue();

    }





    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO SITUACION-----------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */


    public Integer altaSituacion(String descripcion, float relacionGastoIngreso, String idUsuario) {

        Integer idSituacion;

        idSituacion = T_Situacion.altaSituacion(getApplicationContext(), descripcion, relacionGastoIngreso, idUsuario);

        if (conexOnline)
            altaSituacionFB(idSituacion, descripcion, relacionGastoIngreso, idUsuario);

        return idSituacion;
    }


    public void altaSituacionFB(Integer idSituacion, String descripcion,Float relacionGastoIngreso, String idUsuario) {

        objBD = database.getReference("Situacion");

        T_Situacion.Situacion nSituacion = new T_Situacion.Situacion(idSituacion, descripcion, relacionGastoIngreso, idUsuario);
        String item = objBD.child(idSituacion.toString()).getKey();
        objBD.child(item).setValue(nSituacion);
    }

    public void modificaSituacion(Integer idSituacion, String descripcion, float relacionGastoIngreso, String idUsuario) {

        T_Situacion.modificaSituacion(getApplicationContext(), idSituacion, descripcion, relacionGastoIngreso, idUsuario);

        if (conexOnline)
            modificaSituacionFB(idSituacion, descripcion, relacionGastoIngreso, idUsuario);

    }


    public void modificaSituacionFB(Integer idSituacion, String descripcion, Float relacionGastoIngreso, String idUsuario) {
        objBD = database.getReference("Situacion");

        Map<String, T_Situacion.Situacion> nSituacion = new HashMap<>();
        nSituacion.put(idSituacion.toString(), new T_Situacion.Situacion(idSituacion, descripcion, relacionGastoIngreso, idUsuario));

        objBD.setValue(nSituacion);
    }


    public void bajaSituacion(Integer idSituacion) {

        //Se elimina de la base de datos Local y tambien se elimina de FB
        T_Situacion.eliminaSituacion(getApplicationContext(), idSituacion);

        //Se elimina el presupuesto de FB también
        bajaSituacionFB(idSituacion);

    }


    public void bajaSituacionFB(Integer idSituacion) {
        //Eliminar de la BD de FB el presupuesto indicado
        objBD = database.getReference("Situacion");


        objBD.child(idSituacion.toString()).removeValue();

    }


    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO PERIODICIDAD-----------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */

    public void bajaPeriodicidadFB(Integer idPeriodicidad) {
        //Eliminar de la BD de FB la Periodicidad indicada
        objBD = database.getReference("Periodicidad");

        Query query = objBD.orderByChild("idPeriodicidad").equalTo(idPeriodicidad);

        query.addValueEventListener(new ValueEventListener() {
            @Override

            public void onDataChange(DataSnapshot dataSnapshot) {

                DataSnapshot nPeriodicidad = dataSnapshot.getChildren().iterator().next();

                if (nPeriodicidad != null) {
                    String key = nPeriodicidad.getKey();
                    objBD.child(key).removeValue();

                }
            }

            @Override
            public void onCancelled(DatabaseError error) {

            }
        });


    }

    public void altaPeriodicidadFB(Integer idPeriodicidad, String descripcion) {
        objBD = database.getReference("Periodicidad");


        objBD.push().setValue(new T_Periodicidad.Tipo_Periodicidad(idPeriodicidad, descripcion));

    }


    public void modificaPeriodicidadFB(Integer idPeriodicdad, String descripcion) {
        objBD = database.getReference("Periodicidad");

        Map<String, T_Periodicidad.Tipo_Periodicidad> nPeriodicidad = new HashMap<>();
        nPeriodicidad.put(idPeriodicdad.toString(), new T_Periodicidad.Tipo_Periodicidad(idPeriodicdad, descripcion));

        objBD.setValue(nPeriodicidad);
    }




    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------INFO RELATIVA AL OBJETO USUARIO-------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    */


    //Carga desplegable de Alimentacion
    public ArrayAdapter<String> cargaAlimentacion() {
        ArrayAdapter<String> dataAdapter = null;

        //Si no hay conexión se obtiene de BD local
        ArrayList<String> alimentaciones = new ArrayList<String>();
        alimentaciones.add(getResources().getString(R.string.ini_spinner));

        List<T_tipo_Alimentacion.Tipo_Alimentacion> alimentaciones_bd = T_tipo_Alimentacion.obtenerTodosTiposAlimentacion();

        for (int indice = 0; indice < alimentaciones_bd.size(); indice++) {
            alimentaciones.add(alimentaciones_bd.get(indice).getdescripcion());
        }

        dataAdapter = new ArrayAdapter<String>(this, R.layout.support_simple_spinner_dropdown_item, alimentaciones);

        return dataAdapter;
    }


    public Integer altaUsuario(String idUsuario, String pwd, String nombreCompleto, String email, Integer idAlimentacion, String alimentacion, String notificarEmail) {
        Integer codUsuario;

        codUsuario = T_Usuario.altaUsuario(getApplicationContext(), idUsuario, pwd, nombreCompleto, email, idAlimentacion, alimentacion, notificarEmail);

        if (conexOnline)
            altaUsuarioFB(codUsuario, idUsuario, pwd, nombreCompleto, email, idAlimentacion, alimentacion, notificarEmail);

        return codUsuario;
    }


    public void altaUsuarioFB(Integer codUsuario, String idUsuario, String pwd, String nombreCompleto, String email, Integer idAlimentacion, String alimentacion, String notificarEmail) {

        objBD = database.getReference("Usuario");

        T_Usuario.Usuario nUsuario = new T_Usuario.Usuario(codUsuario, idUsuario, pwd, nombreCompleto, email, idAlimentacion, alimentacion, notificarEmail);
        String item = objBD.child(codUsuario.toString()).getKey();
        objBD.child(item).setValue(nUsuario);
    }

    public void modificaUsuario(Integer codUsuario, String idUsuario, String pwd, String nombreCompleto, String email, Integer idAlimentacion, String alimentacion, String notificarEmail) {

        T_Usuario.modificaUsuario(getApplicationContext(), codUsuario, idUsuario, pwd, nombreCompleto, email, idAlimentacion, alimentacion, notificarEmail);

        if (conexOnline)
            modificaUsuarioFB(codUsuario, idUsuario, pwd, nombreCompleto, email, idAlimentacion, alimentacion, notificarEmail);

    }


    public void modificaUsuarioFB(Integer codUsuario, String idUsuario, String pwd, String nombreCompleto, String email, Integer idAlimentacion, String alimentacion, String notificarEmail) {
        objBD = database.getReference("Usuario");

        Map<String, T_Usuario.Usuario> nUsuario = new HashMap<>();
        nUsuario.put(codUsuario.toString(), new T_Usuario.Usuario(codUsuario, idUsuario, pwd, nombreCompleto, email, idAlimentacion, alimentacion, notificarEmail));

        objBD.setValue(nUsuario);
    }


    public void bajaUsuario(Integer codUsuario) {

        //Se elimina de la base de datos Local y tambien se elimina de FB
        T_Usuario.eliminarUsuario(getApplicationContext(), codUsuario);

        //Se elimina el usuario de FB también
        bajaUsuarioFB(codUsuario);

    }


    public void bajaUsuarioFB(Integer codUsuario) {
        //Eliminar de la BD de FB el deseo indicado
        objBD = database.getReference("Usuario");


        objBD.child(codUsuario.toString()).removeValue();

    }

      /*---------------------------------------------------------------------
        ---------------------------------------------------------------------
        -------------INFO RELATIVA AL OBJETO CUENTA_BANCARIA ...-------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
         */


    public Integer altaCuentaBancaria(String descripcion,String numeroCuenta,String idUsuario) {
        Integer idCuenta;

        idCuenta = T_Cuenta_Bancaria.altaCuentaBancaria(getApplicationContext(), descripcion,numeroCuenta,idUsuario);
        if (conexOnline)
            altaCuentaBancariaFB(idCuenta,descripcion,numeroCuenta,idUsuario);

        return idCuenta;
    }


    public void altaCuentaBancariaFB(Integer idCuenta,String descripcion,String numeroCuenta,String idUsuario) {

        objBD = database.getReference("Cuenta_Bancaria");

        T_Cuenta_Bancaria.Cuenta_Bancaria nCuenta = new T_Cuenta_Bancaria.Cuenta_Bancaria (idCuenta,descripcion,numeroCuenta,idUsuario);
        String item = objBD.child(idCuenta.toString()).getKey();
        objBD.child(item).setValue(nCuenta);
    }

    public void modificaCuentaBancaria(Integer idCuenta, String descripcion,String numeroCuenta,String idUsuario) {

        T_Cuenta_Bancaria.modificaCuentaBancaria(getApplicationContext(), idCuenta,descripcion,numeroCuenta,idUsuario);

        if (conexOnline)
            modificaCuentaBancariaFB(idCuenta, descripcion,numeroCuenta,idUsuario);

    }


    public void modificaCuentaBancariaFB(Integer idCuenta, String descripcion,String numeroCuenta,String idUsuario) {

        objBD = database.getReference("Cuenta_Bancaria");

        Map<String, T_Cuenta_Bancaria.Cuenta_Bancaria> ncuenta = new HashMap<>();
        ncuenta.put(idCuenta.toString(), new T_Cuenta_Bancaria.Cuenta_Bancaria(idCuenta, descripcion,numeroCuenta,idUsuario));

        objBD.setValue(ncuenta);
    }
    public void bajaCuentaBancaria(Integer idCuenta) {

        //Se elimina de la base de datos Local y tambien se elimina de FB
        T_Cuenta_Bancaria.eliminaCuentaBancaria(getApplicationContext(), idCuenta);

        //Se elimina el usuario de FB también
        bajaCuentaBancariaFB(idCuenta);

    }


    public void bajaCuentaBancariaFB(Integer idCuenta) {

        //Eliminar de la BD de FB el deseo indicado
        objBD = database.getReference("cuenta_bancaria");


        objBD.child(idCuenta.toString()).removeValue();

    }


}