package edu.uoc.android.gfp.modeloDatos;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteStatement;
import android.os.Build;
import android.support.annotation.RequiresApi;

import com.orm.SugarRecord;
import com.orm.query.Condition;
import com.orm.query.Select;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.android.gfp.utilidades.UtilidadesFecha;

public class T_Deseo {

    public static final List<Deseo> DESEOS = new ArrayList<>();


    //Procedimiento para obtener el total de Deseos en la BD local
    public static List<Deseo> obtenerTodosDeseos() {

        return  Deseo.listAll(Deseo.class);

    }

    //Procedimiento para obtener el total de Deseos en la BD local asociados al usuario
    //que ha hecho Login
    public static List<Deseo> obtenerTodosDeseos(String idUsuario) {

        List<Deseo> Deseos = Deseo.find(Deseo.class, "id_Usuario = ?", idUsuario);
        return Deseos;

    }

    //Procedimiento para comprobar si Deseo ya existe en la BD local
    public static boolean existe(Deseo tDeseo) {

        List<Deseo> Deseo = T_Deseo.Deseo.find(Deseo.class, "id_deseo = ?", String.valueOf(tDeseo.idDeseo));

        if (Deseo == null || Deseo.size() == 0) {
            return false;
        } else {
            return true;
        }

    }

    //Procedimiento para obtener el último idDeseo
    public static Integer ultimoIdDeseo() {

        List<Deseo> ultimoDeseo = Select.from(Deseo.class)
                .orderBy("id_Deseo")
                .list();

        if (ultimoDeseo.size() > 0)
            return ultimoDeseo.get(0).getIdDeseo();
        else
            return 0;

    }

    public static Deseo leerDeseo(Integer idDeseo,String idUsuario){

        List<Deseo> mDeseos = Select.from(Deseo.class)
                .where(Condition.prop("id_Usuario").eq(idUsuario),
                        Condition.prop("id_Deseo").eq(idDeseo))
                .list();


        return mDeseos.get(0);

    }



    public static float obtenerTotalDeseosMensual (Context contexto, String idUsuario, Integer mes, Integer anyo){


        float totalDeseos=0;



        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        SQLiteStatement query = db.compileStatement("SELECT sum(importe)  FROM Deseo WHERE id_Usuario='" + idUsuario + "' and strftime('%m', desde_fecha)='" + mes + "' and strftime('%Y', desde_fecha)='" + anyo + "'");
        try {
            totalDeseos = query.simpleQueryForLong();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            query.close();
        }

        return totalDeseos;
    }

    public static void eliminarDeseo(Context contexto, Integer idDeseo){

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        SQLiteStatement query = db.compileStatement("delete from deseo where id_deseo=" + idDeseo);
        query.execute();
    }

    @RequiresApi(api = Build.VERSION_CODES.N)
    public static Integer altaDeseo(Context contexto, String descripcion, float importe, String desdeFecha,String hastaFecha, String idUsuario ){

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        long idDeseo;


        ContentValues contentValues = new ContentValues();
        contentValues.put("descripcion", descripcion);
        contentValues.put("importe", importe);
        contentValues.put("desde_fecha", desdeFecha);
        contentValues.put("hasta_fecha", hastaFecha);
        contentValues.put("id_Usuario", idUsuario);


        idDeseo=db.insert("Deseo","descripcion,  importe,  desde_fecha,hasta_fecha, id_Usuario",contentValues);

        SQLiteStatement query = db.compileStatement("update deseo set id_Deseo=" + idDeseo + " where id=" + idDeseo);
        query.execute();

        return Math.toIntExact(idDeseo);

    }

    public static void modificaDeseo (Context contexto,Integer idDeseo, String descripcion, float importe, String desdeFecha,String hastaFecha, String idUsuario ){

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        ContentValues contentValues = new ContentValues();
        contentValues.put("descripcion", descripcion);
        contentValues.put("importe", importe);
        contentValues.put("desde_fecha", desdeFecha);
        contentValues.put("hasta_fecha", hastaFecha);
        contentValues.put("id_Usuario", idUsuario);

        db.update("Deseo",contentValues,"id_deseo = ? ", new String[]{idDeseo.toString()});

    }


    public static List<T_Deseo.Deseo> obtenerDetalleDeseos (Context contexto, String descripcion, String dimporte, String himporte, String dfecha, String hfecha, String idUsuario)
    {

        List<T_Deseo.Deseo> mdeseos= new ArrayList<>();
        T_Deseo.Deseo deseo;

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        String consulta;
        String condicion="";
        String separador;


        separador="";

        if (!descripcion.isEmpty())
        {
            condicion= " descipcion ='" + descripcion + "' ";
            separador=" and ";
        }


        if (!dimporte.isEmpty())
        {
            condicion= condicion + separador + " importe >= " + dimporte;
            separador=" and ";
        }


        if (!himporte.isEmpty())
        {
            condicion= condicion + separador + " importe <= " + himporte;
            separador=" and ";
        }

        if (!dfecha.isEmpty())
        {
            condicion= condicion + separador + " desde_fecha >= " + UtilidadesFecha.formatearFecha(dfecha);
            separador=" and ";
        }

        if (!hfecha.isEmpty())
        {
            condicion= condicion + separador + " hasta_fecha <= " + UtilidadesFecha.formatearFecha(hfecha);
            separador=" and ";
        }

        if (!idUsuario.isEmpty())
        {
            condicion= condicion + separador + " id_Usuario = '" + idUsuario +"'";
            separador=" and ";
        }


        consulta=" select * from deseo where " + condicion;

        Cursor c = db.rawQuery(consulta,null);

        c.moveToFirst();

        while (!c.isAfterLast())
        {
            //0 ID INTEGER PRIMARY KEY AUTOINCREMENT ,
            // 1 DESCRIPCION TEXT,
            // 2 DESDE_FECHA TEXT,
            // 3 HASTA_FECHA TEXT,
            // 4 ID_DESEO INTEGER,
            // 5 ID_USUARIO TEXT,
            // 6 IMPORTE FLOAT

            deseo = new T_Deseo.Deseo();
            deseo.idDeseo=Integer.parseInt(c.getString(4));
            deseo.descripcion=c.getString(1);
            deseo.importe=Float.parseFloat(c.getString(6));
            deseo.desdeFecha=UtilidadesFecha.formatearFecha(c.getString(2));
            deseo.hastaFecha=UtilidadesFecha.formatearFecha(c.getString(3));
            deseo.idUsuario=c.getString(5);

            mdeseos.add(deseo); //add the item
            c.moveToNext();
        }


        return mdeseos;
    }


    public static class Deseo extends  SugarRecord{

        public Integer idDeseo;
        public String descripcion;
        public Float importe;
        public String desdeFecha;
        public String hastaFecha;
        public String idUsuario;


        public Deseo() {
            // Default constructor required for calls to DataSnapshot.getValue(User.class)
        }

        public Deseo(Integer idDeseo, String descripcion,Float importe, String desdeFecha, String hastaFecha, String idUsuario) {

            this.idDeseo = idDeseo;
            this.descripcion = descripcion;
            this.importe=importe;
            this.desdeFecha=desdeFecha;
            this.hastaFecha=hastaFecha;
            this.idUsuario=idUsuario;
        }



        public Integer getIdDeseo() {
            return idDeseo;
        }

        public void setIdDeseo(Integer idDeseo) {
            this.idDeseo = idDeseo;
        }

        public String getdescripcion() {
            return descripcion;
        }

        public void setdescripcion(String descripcion) {
            this.descripcion = descripcion;
        }

        public Float getimporte() {
            return importe;
        }

        public void setimporte(float importe) {
            this.importe = importe;
        }

        public String getdesdeFecha() {
            return desdeFecha;
        }

        public void setdesdeFecha(String desdeFecha) {
            this.desdeFecha = desdeFecha;
        }

        public String gethastaFecha() {
            return hastaFecha;
        }

        public void sethastaFecha(String hastaFecha) {
            this.hastaFecha = hastaFecha;
        }

        public String getidUsuario() {
            return idUsuario;
        }

        public void setidUsuario(String idUsuario) {
            this.idUsuario = idUsuario;
        }

        @Override
        public String toString() {

            return "Deseo {" +
                    "idDeseo=" + idDeseo +
                    "descripcion='" + descripcion + '\'' +
                    "importe=" + importe +
                    "desdeFecha='" + desdeFecha + '\'' +
                    "hastaFecha=" + hastaFecha +
                    "idUsuario='" + idUsuario + '\'' +
                    ", descripcion=" + descripcion + '\'' +
                    '}';
        }



    }







}