package edu.uoc.android.gfp.modeloDatos;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteStatement;
import android.os.Build;
import android.support.annotation.RequiresApi;

import com.orm.SugarRecord;
import com.orm.query.Condition;
import com.orm.query.Select;

import java.util.ArrayList;
import java.util.List;

import edu.uoc.android.gfp.utilidades.UtilidadesFecha;

public class T_Presupuesto {

    public static final List<Presupuesto> PRESUPUESTOS = new ArrayList<>();

    //Procedimiento para obtener el total de Presupuestos en la BD local
    public static List<Presupuesto> obtenerTodosPresupuestos() {

        return  Presupuesto.listAll(Presupuesto.class);

    }

    //Procedimiento para obtener el total de presupuestos en la BD local asociados al usuario
    //que ha hecho Login
    public static List<Presupuesto> obtenerTodosPresupuestos(String idUsuario) {

        List<Presupuesto> presupuestos = Presupuesto.find(Presupuesto.class, "id_Usuario = ?", idUsuario);
        return presupuestos;

    }

    //Procedimiento para comprobar si Presupuesto ya existe en la BD local
    public static boolean existe(Presupuesto tPresupuesto) {

        List<Presupuesto> Presupuesto = T_Presupuesto.Presupuesto.find(Presupuesto.class, "id_presupuesto = ?", String.valueOf(tPresupuesto.idPresupuesto));

        if (Presupuesto == null || Presupuesto.size() == 0) {
            return false;
        } else {
            return true;
        }

    }

    //Procedimiento para obtener el último idPresupuesto
    public static Integer ultimoIdPresupuesto() {

        List<Presupuesto> ultimoPresupuesto = Select.from(Presupuesto.class)
                .orderBy("id_Presupuesto")
                .list();

        if (ultimoPresupuesto.size() > 0)
            return ultimoPresupuesto.get(0).getIdPresupuesto();
        else
            return 0;

    }

    public static Presupuesto leerPresupuesto(Integer idPresupuesto,String idUsuario){

        List<Presupuesto> mPresupuestos = Select.from(Presupuesto.class)
                .where(Condition.prop("id_Usuario").eq(idUsuario),
                        Condition.prop("id_Presupuesto").eq(idPresupuesto))
                .list();


        return mPresupuestos.get(0);

    }


    public static float obtenerTotalPresupuestosMensual (Context contexto, String idUsuario, Integer mes, Integer anyo){


        float totalPresupuestos=0;



        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        SQLiteStatement query = db.compileStatement("SELECT sum(importe)  FROM presupuesto WHERE id_Usuario='" + idUsuario + "' and strftime('%m', desde_fecha)='" + mes + "' and strftime('%Y', desde_fecha)='" + anyo + "'");
        try {
            totalPresupuestos = query.simpleQueryForLong();
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            query.close();
        }

        return totalPresupuestos;
    }

    public static void eliminarPresupuesto(Context contexto, Integer idPresupuesto){

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        SQLiteStatement query = db.compileStatement("delete from presupuesto where id_presupuesto=" + idPresupuesto);
        query.execute();
    }

    @RequiresApi(api = Build.VERSION_CODES.N)
    public static Integer altaPresupuesto(Context contexto, String descripcion, float importe, String desdeFecha,String hastaFecha, String idUsuario ){

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        long idPresupuesto;


        ContentValues contentValues = new ContentValues();
        contentValues.put("descripcion", descripcion);
        contentValues.put("importe", importe);
        contentValues.put("desde_fecha", desdeFecha);
        contentValues.put("hasta_fecha", hastaFecha);
        contentValues.put("id_Usuario", idUsuario);


        idPresupuesto=db.insert("Presupuesto","descripcion,  importe,  desde_fecha,hasta_fecha, id_Usuario",contentValues);

        SQLiteStatement query = db.compileStatement("update presupuesto set id_Presupuesto=" + idPresupuesto + " where id=" + idPresupuesto);
        query.execute();

        return Math.toIntExact(idPresupuesto);

    }

    public static void modificaPresupuesto (Context contexto,Integer idPresupuesto, String descripcion, float importe, String desdeFecha,String hastaFecha, String idUsuario ){

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        ContentValues contentValues = new ContentValues();
        contentValues.put("descripcion", descripcion);
        contentValues.put("importe", importe);
        contentValues.put("desde_fecha", desdeFecha);
        contentValues.put("hasta_fecha", hastaFecha);
        contentValues.put("id_Usuario", idUsuario);

        db.update("presupuesto",contentValues,"id_presupuesto = ? ", new String[]{idPresupuesto.toString()});

    }


    public static List<T_Presupuesto.Presupuesto> obtenerDetallePresupuestos (Context contexto, String descripcion, String dimporte, String himporte, String dfecha, String hfecha, String idUsuario)
    {

        List<T_Presupuesto.Presupuesto> mpresupuestos= new ArrayList<>();
        T_Presupuesto.Presupuesto presupuesto;

        SQLiteDatabase db = contexto.openOrCreateDatabase("BDTFM", contexto.MODE_PRIVATE, null);

        String consulta;
        String condicion="";
        String separador;


        separador="";

        if (!descripcion.isEmpty())
        {
            condicion= " descipcion ='" + descripcion + "' ";
            separador=" and ";
        }


        if (!dimporte.isEmpty())
        {
            condicion= condicion + separador + " importe >= " + dimporte;
            separador=" and ";
        }


        if (!himporte.isEmpty())
        {
            condicion= condicion + separador + " importe <= " + himporte;
            separador=" and ";
        }

        if (!dfecha.isEmpty())
        {
            condicion= condicion + separador + " desde_fecha >= " + UtilidadesFecha.formatearFecha(dfecha);
            separador=" and ";
        }

        if (!hfecha.isEmpty())
        {
            condicion= condicion + separador + " hasta_fecha <= " + UtilidadesFecha.formatearFecha(hfecha);
            separador=" and ";
        }

        if (!idUsuario.isEmpty())
        {
            condicion= condicion + separador + " id_Usuario = '" + idUsuario +"'";
            separador=" and ";
        }


        consulta=" select * from presupuesto";

        if (!condicion.isEmpty())
            consulta = consulta + " where " + condicion;

        Cursor c = db.rawQuery(consulta,null);

        c.moveToFirst();

        while (!c.isAfterLast())
        {
            // 0 ID INTEGER PRIMARY KEY AUTOINCREMENT ,
            // 1 DESCRIPCION TEXT,
            // 2 DESDE_FECHA TEXT,
            // 3 HASTA_FECHA TEXT,
            // 4 ID_PRESUPUESTO INTEGER,
            // 5 ID_USUARIO TEXT,
            // 6 IMPORTE FLOAT
            presupuesto = new T_Presupuesto.Presupuesto();
            presupuesto.idPresupuesto=Integer.parseInt(c.getString(4));
            presupuesto.descripcion=c.getString(1);
            presupuesto.importe=Float.parseFloat(c.getString(6));
            presupuesto.desdeFecha=UtilidadesFecha.formatearFecha(c.getString(2));
            presupuesto.hastaFecha=UtilidadesFecha.formatearFecha(c.getString(3));
            presupuesto.idUsuario=c.getString(5);

            mpresupuestos.add(presupuesto); //add the item
            c.moveToNext();
        }


        return mpresupuestos;
    }



    public static class Presupuesto extends SugarRecord {

        public Integer idPresupuesto;
        public String descripcion;
        public float importe;
        public String desdeFecha;
        public String hastaFecha;
        public String idUsuario;


        public Presupuesto() {
            // Default constructor required for calls to DataSnapshot.getValue(User.class)
        }

        public Presupuesto(Integer idPresupuesto,String descripcion,float importe,String desdeFecha,String hastaFecha,String idUsuario) {

            this.idPresupuesto=idPresupuesto;
            this.descripcion=descripcion;
            this.importe=importe;
            this.desdeFecha=desdeFecha;
            this.hastaFecha=hastaFecha;
            this.idUsuario=idUsuario;

        }



        public Integer getIdPresupuesto() {
            return idPresupuesto;
        }

        public void setIdPresupuesto(Integer idPresupuesto) {
            this.idPresupuesto = idPresupuesto;
        }

        public String getdescripcion() {
            return descripcion;
        }

        public void setdescripcion(String descripcion) {
            this.descripcion = descripcion;
        }

        public Float getimporte() {
            return importe;
        }

        public void setimporte(float importe) {
            this.importe = importe;
        }

        public String getdesdeFecha() {
            return desdeFecha;
        }

        public void setdesdeFecha(String desdeFecha) {
            this.desdeFecha = desdeFecha;
        }

        public String gethastaFecha() {
            return hastaFecha;
        }

        public void sethastaFecha(String hastaFecha) {
            this.hastaFecha = hastaFecha;
        }

        public String getidUsuario() {
            return idUsuario;
        }

        public void setidUsuario(String idUsuario) {
            this.idUsuario = idUsuario;
        }

        @Override
        public String toString() {

            return "Presupuesto {" +
                    "idPresupuesto=" + idPresupuesto +
                    "descripcion='" + descripcion + '\'' +
                    "importe=" + importe +
                    "desdeFecha='" + desdeFecha + '\'' +
                    "hastaFecha=" + hastaFecha +
                    "idUsuario='" + idUsuario + '\'' +
                    ", descripcion=" + descripcion + '\'' +
                    '}';
        }



    }







}