package edu.uoc.android.gfp.vistas;

import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;

import java.text.DecimalFormat;

import edu.uoc.android.gfp.R;
import edu.uoc.android.gfp.modeloDatos.T_Gasto;
import edu.uoc.android.gfp.modeloDatos.T_Ingreso;
import edu.uoc.android.gfp.utilidades.UtilidadesFecha;
import edu.uoc.android.gfp.vistas.Detalles.DetalleCompleto;
import edu.uoc.android.gfp.vistas.Informes.detalleInformes;

public class Bienvenida extends AppCompatActivity implements View.OnClickListener {

    //Parámetros de la actividad
    public String idUsuario;



    //Elementos de la interfaz
    private TextView eti_gastos;
    private TextView eti_ingresos;
    private TextView eti_titulo;
    private TextView eti_balance;
    private ImageView img_balance;
    private Button btnDetalle, btnInformes;




    @Override
    protected void onCreate(Bundle savedInstanceState) {

        super.onCreate(savedInstanceState);
        setContentView(R.layout.bienvenida);


        //Obtenemos el usuario que ha hecho Login en la app
        idUsuario= getIntent().getExtras().getString("idUsuario");

        if (savedInstanceState == null) {
            //Fragmento para incluir la cabecera
            Bundle arguments = new Bundle();
            arguments.putString(FragmentoCabecera.arg_idUsuario, idUsuario);
            arguments.putString(FragmentoCabecera.arg_funcionalidad, "Bienvenida");
            FragmentoCabecera fragment = new FragmentoCabecera();
            fragment.setArguments(arguments);
            getSupportFragmentManager().beginTransaction()
                    .add(R.id.frame_cabecera, fragment)
                    .commit();
        }

        UtilidadesFecha utilFecha = new UtilidadesFecha();
        utilFecha.obtenerMesAnyoActual();

        eti_titulo =  (TextView) findViewById(R.id.eti_titulo);
        eti_titulo.setText(utilFecha.mesActualLetra + " " +  utilFecha.anyoActual.toString());


        DecimalFormat df = new DecimalFormat("###.##");

        eti_ingresos = (TextView) findViewById(R.id.t_ingresos);
        eti_gastos =  (TextView) findViewById(R.id.t_gastos);
        eti_balance =  (TextView) findViewById(R.id.t_balance);
        img_balance =  (ImageView) findViewById(R.id.img_balance);
        btnDetalle = (Button) findViewById(R.id.btn_detalle_mensual);
        btnInformes = (Button) findViewById(R.id.btn_informes);


        // Metodos Listener
        btnDetalle.setOnClickListener((View.OnClickListener) this);
        btnInformes.setOnClickListener((View.OnClickListener) this);

        float totalGastos;
        float totalIngresos;
        float balance;

        totalIngresos=obtenerTotalIngresosMensual(utilFecha.mesActualNumero,utilFecha.anyoActual);
        totalGastos=obtenerTotalGastosMensual(utilFecha.mesActualNumero,utilFecha.anyoActual);

        eti_ingresos.setText(String.valueOf(df.format(totalIngresos)) + " €");
        eti_gastos.setText(String.valueOf(df.format(totalGastos))+ " €");

        balance=totalIngresos-totalGastos;

        eti_balance.setText("Saldo " + String.valueOf(df.format(balance))+ " €");

        if (balance>0)
        {
            //eti_balance.setTextColor();
            img_balance.setImageResource(R.drawable.monedas);

        }
        else
        {
            //eti_balance.setTextColor();
            img_balance.setImageResource(R.drawable.advertencia);

        }


    }



    /*-------------------------------------------------------------------
  ---------------------------------------------------------------------
  -------------OBTENER TOTAL GASTOS MES EN CURSO-----------------------
  ---------------------------------------------------------------------
  ---------------------------------------------------------------------
  -------------------------------------------------------------------*/
    private float obtenerTotalGastosMensual(Integer mes,Integer anyo)
    {
        float totalGastos;

        totalGastos = T_Gasto.obtenerTotalGastosMensual(getApplicationContext(),idUsuario,mes,anyo);
        return totalGastos;

    }

    /*-----------------------------------------------------------------
  ---------------------------------------------------------------------
  -------------OBTENER TOTAL INGRESOS MES EN CURSO---------------------
  ---------------------------------------------------------------------
  ---------------------------------------------------------------------
  -------------------------------------------------------------------*/
    private float obtenerTotalIngresosMensual(Integer mes, Integer anyo)
    {
        float totalIngresos;

        totalIngresos = T_Ingreso.obtenerTotalIngresosMensual(getApplicationContext(),idUsuario,mes,anyo);
        return totalIngresos;

    }




  /*-------------------------------------------------------------------
  ---------------------------------------------------------------------
  -------------GESTIÓN DE LOS BOTONES DEL BIENVENIDA--- ---------------
  ---------------------------------------------------------------------
  ---------------------------------------------------------------------
  -------------------------------------------------------------------*/
    @Override
    public void onClick( View v) {

        Intent intent=null;

        // Dependiendo si se pulsa Login o Registro ejecuta una actividad u otra
        switch (v.getId()) {
            case R.id.btn_detalle_mensual:
                intent = new Intent(this, DetalleCompleto.class);
                intent.putExtra("idUsuario", idUsuario);
                startActivity(intent);
                break;
            case R.id.btn_informes:
                intent = new Intent(this, detalleInformes.class);
                intent.putExtra("idUsuario", idUsuario);
                startActivity(intent);
                break;
        }
    }

}
