package edu.uoc.android.gfp.vistas.Contribucion;

import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.view.Gravity;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;


import edu.uoc.android.gfp.MyApplication;
import edu.uoc.android.gfp.R;
import edu.uoc.android.gfp.utilidades.UtilidadesTeclado;
import edu.uoc.android.gfp.utilidades.UtilidadesFecha;
import edu.uoc.android.gfp.vistas.Bienvenida;
import edu.uoc.android.gfp.vistas.FragmentoCabecera;


public class RegistroContribucion extends AppCompatActivity implements View.OnClickListener {


    // Variables de los elementos de la pantalla
    private EditText txtDescripcion, txtImporte, txtFecha;

    private Button btnCancelar, btnGuardar;
    private Spinner despDeseo;


    //Variables necesarias para registrar un Contribucion
    public String descripcion;
    public Integer idDeseo=-1;
    public float importe;
    public String fechaContribucion ;
    public String idUsuario;
    public String deseo="";
    public Integer idContribucion;

    //Para la gestión de ocultar el teclado una vez que un campo de texto pierde el foco
    public UtilidadesTeclado utilTecla;
    public UtilidadesFecha utilFecha;



    public MyApplication myApplication;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        {

            super.onCreate(savedInstanceState);
            setContentView(R.layout.registrocontribucion);

            idUsuario= getIntent().getExtras().getString("idUsuario");

            if (savedInstanceState == null) {
                //Fragmento para incluir la cabecera
                Bundle arguments = new Bundle();
                arguments.putString(FragmentoCabecera.arg_idUsuario, idUsuario);
                arguments.putString(FragmentoCabecera.arg_funcionalidad, "Alta Nueva Contribucion");
                FragmentoCabecera fragment = new FragmentoCabecera();
                fragment.setArguments(arguments);
                getSupportFragmentManager().beginTransaction()
                        .add(R.id.frame_cabecera, fragment)
                        .commit();
            }

            //Para la gestión de ocultar el teclado una vez que un campo de texto pierde el foco
            utilTecla = new UtilidadesTeclado(this);
            utilFecha = new UtilidadesFecha();

            //Inicialización de todos los elementos de la pantalla

            txtDescripcion = findViewById(R.id.t_descripcion);
            txtDescripcion.setOnFocusChangeListener(new View.OnFocusChangeListener() {
                @Override
                public void onFocusChange(View v, boolean hasFocus) {
                    if(!hasFocus)
                    {
                        utilTecla.hideKeyBoard(v);
                    }
                }
            });


            txtImporte = findViewById(R.id.t_importe);
            txtImporte.setOnFocusChangeListener(new View.OnFocusChangeListener() {
                @Override
                public void onFocusChange(View v, boolean hasFocus) {
                    if(!hasFocus)
                    {
                        utilTecla.hideKeyBoard(v);
                    }
                }
            });

            txtFecha = findViewById(R.id.t_fecha);
            txtFecha.setOnFocusChangeListener(new View.OnFocusChangeListener() {
                @Override
                public void onFocusChange(View v, boolean hasFocus) {
                    if(!hasFocus)
                    {
                        if (!txtFecha.getText().toString().isEmpty())
                        {
                            if (!UtilidadesFecha.validarFecha(txtFecha.getText().toString())) {
                                //La validación de la fecha no ha sido correcta
                                //mostramos mensaje
                                Toast.makeText(RegistroContribucion.this, "Formato de fecha incorrecto mm/dd/yyyy", Toast.LENGTH_SHORT).show();

                            }
                        }
                        utilTecla.hideKeyBoard(v);
                    }
                }
            });

            btnCancelar = findViewById(R.id.b_cancelar);
            btnGuardar = findViewById(R.id.b_guardar);

            // Metodos Listener
            btnCancelar.setOnClickListener(this);
            btnGuardar.setOnClickListener(this);


            despDeseo = findViewById(R.id.t_deseo);


            myApplication = (MyApplication) getApplicationContext();


            // Carga desplegable de tipo Gasto
            despDeseo.setAdapter(myApplication.cargaDeseo(idUsuario));

            despDeseo.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {

                @Override
                public void onItemSelected(AdapterView<?> arg0, View arg1,
                                           int pos, long id) {

                    String descripcion = arg0.getItemAtPosition(pos)
                            .toString();

                    if (descripcion.equals(getResources().getString(R.string.ini_spinner))) {
                        //No se entiende seleccionado ningún valor
                        deseo="";
                        idDeseo = -1;
                    } else {
                        deseo=descripcion;
                        idDeseo = pos;


                    }

                }

                @Override
                public void onNothingSelected(AdapterView<?> arg0) {
                    // TODO Auto-generated method stub

                }
            });

        }


    }


    @Override
    public void onClick (View v){

        Boolean continua;

        continua=true;

        // Dependiendo si se pulsa Login o Registro ejecuta una actividad u otra
        switch (v.getId()) {
            case R.id.b_guardar:

             if (txtDescripcion.toString().isEmpty())
                    continua=false;

                if (txtImporte.getText().toString().isEmpty())
                    continua=false;

                if (txtFecha.getText().toString().isEmpty())
                    continua=false;

                if (idDeseo==-1)
                    continua=false;


                if (continua==true)
                {


                    descripcion=txtDescripcion.getText().toString();

                    importe= Float.parseFloat(txtImporte.getText().toString());
                    fechaContribucion= UtilidadesFecha.formatearFecha(txtFecha.getText().toString());

                    idContribucion=myApplication.altaContribucion(descripcion, idDeseo, importe, fechaContribucion, idUsuario,deseo);

                    //Una vez se ha registadrado el gasto se redirige a la ficha del gasto
                    Intent intent_mg = new Intent(getApplicationContext(), FichaContribucion.class);
                    intent_mg.putExtra("arg_idUsuario", idUsuario);
                    intent_mg.putExtra("arg_idContribucion", idContribucion);
                    startActivity(intent_mg);


                }
                else
                {
                    Toast toast1 = Toast.makeText(getApplicationContext(), "Existen campos vacios. Revise el formulario", Toast.LENGTH_SHORT);
                    toast1.setGravity(Gravity.CENTER,0,0);
                    toast1.show();
                }
                break;
            case R.id.b_cancelar:
                //Se vuelve a la página de bienvenida
                Intent intent = new Intent(this, Bienvenida.class);
                intent.putExtra("idUsuario", idUsuario);
                startActivity(intent);
                break;
            default:
                break;

        }
    }

    public void limpiarCampos()
    {

    }
}