package edu.uoc.android.gfp.vistas;

import android.app.Activity;
import android.content.Intent;
import android.support.annotation.NonNull;
import android.os.Bundle;
import android.view.Gravity;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.firebase.auth.AuthResult;
import com.google.firebase.auth.FirebaseAuth;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.Query;
import com.google.firebase.database.ValueEventListener;

import edu.uoc.android.gfp.R;
import edu.uoc.android.gfp.modeloDatos.T_Usuario;
import edu.uoc.android.gfp.utilidades.UtilidadesTeclado;
import edu.uoc.android.gfp.vistas.usuario.RegistroUsuario;

public class Login extends Activity implements View.OnClickListener {

    //Propiedad para conectarse a Firebase de google
    private FirebaseAuth mAuth;
    private FirebaseDatabase database;

    //Variable que utilizamos para indicar si hay conexión a Internet
    private boolean conexOnline;

    //Propiedad para BD
    private DatabaseReference objBD;

    //Variables para la entrada de los datos del usuario

    // Variables de los elementos de la pantalla
    private EditText txtUsuario, txtPassword;
    private Button btnLogin, btnRegistro;

    private  Boolean validado;


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.login);


        conexOnline=false;

        /*-------------------------------------------------------------------
        ---------------------------------------------------------------------
        -------------INFO RELATIVA A LA CONEXIÓN CON LA BD-------------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
         */

        mAuth = FirebaseAuth.getInstance();
        database = FirebaseDatabase.getInstance();

        //Autenticación contra la BD de firebase
        mAuth.signInWithEmailAndPassword("fgg2@alu.ua.es", "12345678")
                .addOnCompleteListener(new OnCompleteListener<AuthResult>() {
                    @Override
                    public void onComplete(@NonNull Task<AuthResult> task) {

                        if (!task.isSuccessful()) {
                            //Si no hay conexión con el servidor enviamos una notificación a la app
                            //Indicando que la señal ha fallado
                            Toast.makeText(Login.this, "Sin conexión", Toast.LENGTH_SHORT).show();
                            conexOnline=false;
                        }
                        else
                            conexOnline=true;
                    }
                });



        /*-------------------------------------------------------------------
        ---------------------------------------------------------------------
        -------------INFO RELATIVA A LOS DATOS DE ACCESO --------------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
        ---------------------------------------------------------------------
         */

        final UtilidadesTeclado utilTecla = new UtilidadesTeclado(this);

        // Campos de entrada del texto
        txtUsuario = (EditText) findViewById(R.id.t_usuario);
        txtPassword = (EditText) findViewById(R.id.t_password);

        txtPassword.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if(!hasFocus)
                {
                    utilTecla.hideKeyBoard(v);
                }
            }
        });

        txtUsuario.setOnFocusChangeListener(new View.OnFocusChangeListener() {
            @Override
            public void onFocusChange(View v, boolean hasFocus) {
                if(!hasFocus)
                {
                    utilTecla.hideKeyBoard(v);
                }
            }
        });

        // Botones
        btnLogin = (Button) findViewById(R.id.b_login);
        btnRegistro = (Button) findViewById(R.id.b_registro);


        // Metodos Listener
        btnLogin.setOnClickListener((View.OnClickListener) this);
        btnRegistro.setOnClickListener((View.OnClickListener) this);




    }



    /*-------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------GESTIÓN DE LOS BOTONES DEL LOGIN--- --------------------
    ---------------------------------------------------------------------
    ---------------------------------------------------------------------
    -------------------------------------------------------------------*/
    @Override
    public void onClick( View v) {
        // Dependiendo si se pulsa Login o Registro ejecuta una actividad u otra
        switch (v.getId()) {
            case R.id.b_login:
                if (conexOnline) {

                    validarUsuario(txtUsuario.getText().toString(),txtPassword.getText().toString());

                }

                else
                {

                    if (T_Usuario.validarUsuarioBD(txtUsuario.getText().toString(),txtPassword.getText().toString()))
                        irBienvenida();
                    else {
                        Toast toast1 = Toast.makeText(getApplicationContext(), "No existe un usuario con los datos de acceso indicados", Toast.LENGTH_SHORT);
                        toast1.setGravity(Gravity.CENTER, 0, 0);
                        toast1.show();
                    }
                }
                break;
            case R.id.b_registro:

                Intent i = new Intent(this, RegistroUsuario.class);
                startActivity(i);
                break;

            default:
                break;

        }
    }


    private void validarUsuario (String usuario,  final String  contrasenya) {


        if (conexOnline) {

            // Existe conexión a internet el usuario se valida contra FB
            ValueEventListener eventListener;

            objBD = database.getReference("Usuario");

            Query query1 = objBD.orderByChild("idUsuario").equalTo(usuario).limitToFirst(1);
            query1.addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot dataSnapshot) {


                    for (DataSnapshot userSnapshot : dataSnapshot.getChildren()) {
                        T_Usuario.Usuario user = userSnapshot.getValue(T_Usuario.Usuario.class);

                        if (contrasenya.equals(user.getpwd())) {
                            //Usuario correcto
                            Toast.makeText(Login.this, "USUARIO CORRECTO", Toast.LENGTH_SHORT).show();
                            irBienvenida();
                        } else {
                            //Usuario incorrecto
                            Toast.makeText(Login.this, "USUARIO INCORRECTO", Toast.LENGTH_SHORT).show();

                        }

                    }

                }

                @Override
                public void onCancelled(DatabaseError databaseError) {
                    //Se ocorrer um erro
                }
            });
        }
        else
        {
            //No existe conexión a intenert el usuario se valida contra la BD Local
           if(T_Usuario.validarUsuarioBD(usuario,contrasenya))
           {
               //Usuario correcto
               Toast.makeText(Login.this, "USUARIO CORRECTO (VALIDACIÓN LOCAL)", Toast.LENGTH_LONG).show();
               irBienvenida();
           }
            else
           {
               //Usuario incorrecto
               Toast.makeText(Login.this, "USUARIO INCORRECTO (VALIDACIÓN LOCAL)", Toast.LENGTH_SHORT).show();
           }
        }

    }

    private void irBienvenida ()
    {
        //Si la validación es correcta acceso a la pantalla de Bienvenida
        Intent intent = new Intent(this, Bienvenida.class);
        intent.putExtra("idUsuario", txtUsuario.getText().toString());
        startActivity(intent);

    }

}


